from typing import Optional, List, Union, TYPE_CHECKING

from ._serializer import Serializer
from ._tenor_bid_ask import TenorBidAsk
from ._tenor_types import TenorTypes

if TYPE_CHECKING:
    from .._types import OptStr, StrStrings


class FxSwapPoints(Serializer):
    """
    An object that contains data about the source (contributor)
    of the cross currency forward, the tenor types used in the valuation and
    the possible swap points override

    Parameters
    ----------
    additional_tenor_types : list of str, TenorTypes, list of TenorTypes, optional
        An array of tenor types used in the valuation
    source : str, optional
        The Refinitiv code of the contributor (e.g., ‘ICAP’).
        For the Refinitiv composite source, use ‘Composite’, or ignore the property
    overrides : dict, TenorBidAsk, list of TenorBidAsk, optional
        An array of tenors and corresponding swap bid, ask prices
    """

    def __init__(
        self,
        additional_tenor_types: Optional[
            Union["StrStrings", TenorTypes, List[TenorTypes]]
        ] = None,
        source: "OptStr" = None,
        overrides: Optional[
            Union[dict, List[dict], TenorBidAsk, List[TenorBidAsk]]
        ] = None,
    ) -> None:
        super().__init__()
        self.additional_tenor_types = additional_tenor_types
        self.source = source
        self.overrides = overrides

    @property
    def additional_tenor_types(self):
        return self._get_list_of_enums("additionalTenorTypes")

    @additional_tenor_types.setter
    def additional_tenor_types(self, value):
        self._set_list_of_enums(TenorTypes, "additionalTenorTypes", value)

    @property
    def overrides(self):
        return self._get_list_param("overrides")

    @overrides.setter
    def overrides(self, value):
        self._set_list_param("overrides", value)

    @property
    def source(self):
        return self._get_param("source")

    @source.setter
    def source(self, value):
        self._set_param("source", value)



from typing import Callable, TYPE_CHECKING

import pandas as pd

from ._retrieval_data_provider import (
    ListOfFile,
    DownloadFileError,
    DownloadAllFileResponse,
)
from .retrieval import Definition as RetrievalDefinition
from .._content_data import Data
from .._content_data_provider import ContentDataProvider
from .._content_response_factory import ContentResponseFactory
from .._error_parser import ErrorParser
from .._universe_content_validator import UniverseContentValidator
from ...delivery._data._data_provider import (
    RequestFactory,
)

if TYPE_CHECKING:
    from ...delivery._data._data_provider import ParsedData


class FilingsSearchRequestFactory(RequestFactory):
    def get_body_parameters(self, *_, **kwargs) -> dict:
        query = kwargs.get("query", "")
        variables = kwargs.get("variables", {})
        return {"query": query, "variables": variables}


class FilingsSearchFile:
    def __init__(
        self,
        title: str = None,
        filename: str = None,
        mimetype: str = None,
        dcn: str = None,
        doc_id: str = None,
        filing_id: int = None,
    ):
        """
        Parameters
        ----------
        title : str
            Document title of the file
        filename : str
            Name of the file
        mimetype : str
            Mime type of the file
        dcn: str
            DCN of the file
        doc_id: str
            Doc ID of the file
        filing_id: int
            Financial Filing ID
        """
        self.filename = filename
        self.title = title
        self.mimetype = mimetype
        self.dcn = dcn
        self.doc_id = doc_id
        self.filing_id = filing_id

    def _download_file(self):
        response = None
        error = False
        if pd.notna(self.filename) and self.filename:
            response = RetrievalDefinition(filename=self.filename).get_data()
        elif pd.notna(self.dcn) and self.dcn:
            response = RetrievalDefinition(dcn=self.dcn).get_data()
        elif pd.notna(self.doc_id) and self.doc_id:
            response = RetrievalDefinition(doc_id=self.doc_id).get_data()
        elif pd.notna(self.filing_id) and self.filing_id:
            response = RetrievalDefinition(filing_id=str(self.filing_id)).get_data()
        else:
            error = True
        return response, error

    def download(self, path: str = None) -> DownloadAllFileResponse:
        """

        Parameters
        ----------
        path : str
            Destination of the download file. Default is current working directory.

        Returns
        -------
        DownloadAllFileResponse
        """
        response, err = self._download_file()
        if err:
            raise DownloadFileError(
                code=None,
                message=f"Cannot download file. Missing one of Filename, DCN, DocID and Filing ID",
            )
        return response.data.files.download(path=path)

    async def download_async(
        self, path: str = None, callback: Callable = None
    ) -> DownloadAllFileResponse:
        """

        Parameters
        ----------
        path : str
            Destination of the download file. Default is current working directory.
        callback: Callable
            Callback function will be called after the process is completed

        Returns
        -------
        DownloadAllFileResponse
        """
        response, err = self._download_file()
        if err:
            errors = [
                DownloadFileError(
                    code=None,
                    message=f"Cannot download file. Missing one of Filename, DCN, DocID and Filing ID",
                )
            ]
            return DownloadAllFileResponse(files=[self], errors=errors)
        return await response.data.files.download_async(path=path, callback=callback)


class FilingsSearchData(Data):
    _files = None

    def _get_data_df(self):
        data_df = []
        for row in self._raw["data"].get("FinancialFiling"):
            filing_document = row.get("FilingDocument", {})
            identifiers = filing_document.get("Identifiers", [])
            dcn = ""
            if len(identifiers) > 0:
                dcn = identifiers[0].get("Dcn", "")
            doc_id = filing_document.get("DocId", "")
            financial_filing_id = filing_document.get("FinancialFilingId", "")
            document_title = filing_document.get("DocumentSummary", {}).get(
                "DocumentTitle"
            )
            filenames = []
            if filing_document.get("FilesMetaData"):
                filenames = filing_document.get("FilesMetaData", {})
            if len(filenames) == 0:
                data_df.append(
                    [
                        document_title,
                        None,
                        None,
                        dcn,
                        doc_id,
                        financial_filing_id,
                    ]
                )
            for filename in filenames:
                data_df.append(
                    [
                        document_title,
                        filename.get("FileName"),
                        filename.get("MimeType"),
                        dcn,
                        doc_id,
                        financial_filing_id,
                    ]
                )
        return data_df

    @property
    def df(self):
        if self._dataframe is None and self._raw and "errors" not in self._raw:
            # generate headers for df
            headers = [
                {"title": "DocumentTitle", "type": "string"},
                {"title": "Filename", "type": "string"},
                {"title": "MimeType", "type": "string"},
                {"title": "Dcn", "type": "string"},
                {"title": "DocId", "type": "string"},
                {"title": "FinancialFilingId", "type": "int"},
            ]
            data_df = []
            if self._raw.get("data") is not None:
                data_df = self._get_data_df()
            data_df = {"headers": headers, "data": data_df}
            self._dataframe = self._dfbuilder(data_df, **self._kwargs)
        return self._dataframe

    @property
    def files(self):
        """
        Returns
        -------
        ListOfFile[FilingsSearchFile]
        """
        if self._files is None:
            self._files = ListOfFile()
            if self.df is not None and not self.df.empty:
                self._files.extend(FilingsSearchFile(*row) for row in self.df.values)
        return self._files


class FilingsSearchValidator(UniverseContentValidator):
    @classmethod
    def content_data_has_no_errors(cls, data: "ParsedData") -> bool:
        if data.content_data.get("errors"):
            data.error_codes = None
            data.error_messages = data.content_data["errors"][0].get("message")
            return False

        return True

    def __init__(self) -> None:
        super().__init__()
        self.validators.append(self.content_data_has_no_errors)


filings_search_data_provider = ContentDataProvider(
    request=FilingsSearchRequestFactory(),
    response=ContentResponseFactory(data_class=FilingsSearchData),
    validator=FilingsSearchValidator(),
    parser=ErrorParser(),
)



import json

import allure
import pytest

import refinitiv.data.content.ipa.financial_contracts as rdf
from refinitiv.data.errors import RDError
from tests.integration.content.ipa.contracts.swap.conftest import (
    check_http_status_is_success_and_df_value_not_empty,
    check_stream_state_and_df_from_stream,
    swap_definition_01,
    swap_definition_02,
    swap_universe,
)
from tests.integration.helpers import (
    get_async_response_from_definitions,
    check_dataframe_column_date_for_datetime_type,
)


@allure.suite("Content object - SWAP")
@allure.feature("Content object - SWAP")
@allure.severity(allure.severity_level.CRITICAL)
class TestSwap:
    @allure.title("Create swap definition object with valid params and get data")
    @pytest.mark.caseid("36762994")
    def test_get_swap_analytics_without_fields(self, open_session):
        response = swap_definition_02().get_data()
        check_http_status_is_success_and_df_value_not_empty(response)
        check_dataframe_column_date_for_datetime_type(response)

    @allure.title("Create swap definition object with legs and get data")
    @pytest.mark.caseid("36763041")
    def test_get_swap_analytics_with_legs(self, open_session):
        response = swap_definition_01().get_data()
        check_http_status_is_success_and_df_value_not_empty(response)

    @allure.title("Create swap definition object without any params and get data")
    @pytest.mark.caseid("36763075")
    def test_get_swap_analytics_with_empty_definition(self, open_session):
        with pytest.raises(RDError) as error:
            rdf.swap.Definition().get_data()
        assert (
            str(error.value)
            == "Error code QPS-DPS.5004 | Invalid input: 'tenor' not specified or incorrect. Cannot extract Tenor value from instrumentCode."
        )

    @allure.title("Create swap definition object asynchronously")
    @pytest.mark.caseid("36763077")
    async def test_get_swap_analytics_asynchronously(self, open_session_async):
        response_01, response_02 = await get_async_response_from_definitions(
            swap_definition_01(), swap_definition_02()
        )
        check_http_status_is_success_and_df_value_not_empty(response_02)

    @allure.title("Create swap stream")
    @pytest.mark.caseid("36763080")
    def test_create_swap_stream(self, open_session):
        stream = swap_definition_01().get_stream()
        stream.open()

        check_stream_state_and_df_from_stream(stream)

    @allure.title("Create a swap definition with extended parameters")
    @pytest.mark.caseid("C37561644")
    def test_get_swap_with_extended_params(self, open_session):
        response = rdf.swap.Definition(
            tenor="5Y",
            legs=[
                rdf.swap.LegDefinition(
                    index_tenor="1Y",
                    interest_type=rdf.swap.InterestType.FLOAT,
                    interest_payment_frequency=rdf.swap.Frequency.QUARTERLY,
                    direction=rdf.swap.Direction.RECEIVED,
                    notional_ccy="GBP",
                ),
            ],
            extended_params={
                "instrumentDefinition": {
                    "InstrumentTag": "my tag",
                    "tenor": "2Y",
                    "legs": [
                        {
                            "direction": "Paid",
                            "interestType": "Float",
                            "notionalCcy": "EUR",
                            "notionalAmount": 1,
                            "interestPaymentFrequency": "Quarterly",
                        },
                        {
                            "direction": "Received",
                            "interestType": "Float",
                            "notionalCcy": "EUR",
                            "indexTenor": "5Y",
                            "interestPaymentFrequency": "Quarterly",
                        },
                    ],
                },
                "pricingParameters": {
                    "indexConvexityAdjustmentIntegrationMethod": "RiemannSum",
                    "indexConvexityAdjustmentMethod": "BlackScholes",
                    "valuationDate": "2020-06-01",
                },
            },
        ).get_data()

        check_http_status_is_success_and_df_value_not_empty(response)

        request = json.loads(response.request_message.content.decode("utf-8"))
        request_universe = request.get("universe")[0]
        assert (
            request_universe == swap_universe
        ), f"Extended params are applied incorrectly"
        assert response.data.df.InstrumentTag[0] == "my tag"



from refinitiv.data._qpl import FxSpotQuote


def test_fx_spot_quote():
    ask = 1.10
    bid = 2.20
    source = "source"
    spot_quote = FxSpotQuote(source, ask, bid)

    assert spot_quote.ask == ask
    assert spot_quote.bid == bid
    assert spot_quote.source == source

    assert spot_quote.get_dict() == {"ask": 1.1, "bid": 2.2, "source": "source"}



import base64
from os import makedirs
from os.path import join as join_path, exists
from typing import List

from ...._tools import get_from_path, cached_property


class Image:
    def __init__(self, data):
        self._data = data

    @cached_property
    def _image(self):
        file_base_64 = get_from_path(self._data, "newsItem/OtherContent/0/_", delim="/")
        return base64.b64decode(file_base_64)

    def save(self, path: str = None):
        """
        Parameters
        ----------
        path : str, optional
            Path to save file. Default is current working directory.

        """
        if path and not exists(path):
            makedirs(path)
        filename = self.filename
        if path:
            filename = join_path(path, filename)
        with open(filename, "wb+") as f:
            f.write(self._image)

    def show(self) -> "Union(Image, IPythonImage)":
        try:
            from IPython.display import Image as IPythonImage

            return IPythonImage(data=self._image)
        except ImportError:
            return self

    @property
    def size(self) -> int:
        return len(self._image)

    @cached_property
    def filename(self) -> str:
        return get_from_path(
            self._data,
            "newsItem/StoryProps/0/ContentMeta/0/HeadlineText/0/_",
            delim="/",
        )

    @cached_property
    def provider(self) -> "List[str]":
        return get_from_path(
            self._data,
            "newsItem/StoryProps/0/ItemMeta/0/Provider",
            delim="/",
        )

    @cached_property
    def body_type(self) -> "List[str]":
        return get_from_path(
            self._data,
            "newsItem/StoryProps/0/ItemMeta/0/BodyType",
            delim="/",
        )

    @cached_property
    def source(self) -> "List[str]":
        return get_from_path(
            self._data,
            "newsItem/StoryProps/0/ItemMeta/0/Source",
            delim="/",
        )

    @cached_property
    def version_created(self) -> "List[str]":
        return get_from_path(
            self._data,
            "newsItem/StoryProps/0/ItemMeta/0/VersionCreated",
            delim="/",
        )

    @cached_property
    def first_created(self) -> "List[str]":
        return get_from_path(
            self._data,
            "newsItem/StoryProps/0/ItemMeta/0/FirstCreated",
            delim="/",
        )

    @cached_property
    def available_rsf(self) -> "List[str]":
        return get_from_path(
            self._data,
            "newsItem/StoryProps/0/ItemMeta/0/AvailableRSF",
            delim="/",
        )


class ResizedImage(Image):
    @cached_property
    def _image(self):
        return self._data.get("image")

    @cached_property
    def filename(self):
        return self._data.get("filename")



import pytest

from refinitiv.data.content import symbol_conversion
from refinitiv.data.content.symbol_conversion import (
    SymbolTypes,
    CountryCode,
    AssetClass,
    AssetState,
)
from refinitiv.data.content.symbol_conversion._definition import DEFAULT_SCOPE
from refinitiv.data.content.symbol_conversion._symbol_type import SYMBOL_TYPE_VALUES


def test_definition_repr():
    # given
    symbols_list = ["MSFT.O", "AAPL.O", "GOOG.O"]
    kwargs = {
        "symbols": symbols_list,
        "from_symbol_type": SymbolTypes.RIC,
        "to_symbol_types": None,
        "extended_params": {},
    }

    # when
    definition = symbol_conversion.Definition(**kwargs)

    # then
    assert "{symbols='MSFT.O,AAPL.O,GOOG.O'}" in repr(definition)


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        ("US5949181045", "US5949181045"),
        (["US5949181045", "US02079K1079"], "US5949181045,US02079K1079"),
        ([""], ""),
        ([], None),
        (None, None),
        ("", None),
    ],
)
def test_symbols(input_value, expected_value):
    definition = symbol_conversion.Definition(input_value)

    assert definition.symbols == expected_value


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        ("RIC", "RIC"),
        (SymbolTypes.RIC, "RIC"),
        (["RIC", "IssueISIN"], "['RIC', 'IssueISIN']"),
        ([SymbolTypes.RIC, SymbolTypes.ISIN], "['RIC', 'IssueISIN']"),
        ("", "_AllUnique"),
        ([], "_AllUnique"),
        (None, "_AllUnique"),
    ],
)
def test_from_symbol_type(input_value, expected_value):
    definition = symbol_conversion.Definition("", from_symbol_type=input_value)

    assert definition.from_symbol_type == expected_value


def test_from_symbol_type_default_value():
    definition = symbol_conversion.Definition("")

    assert definition.from_symbol_type == DEFAULT_SCOPE


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        ("RIC", "DocumentTitle,RIC"),
        (SymbolTypes.RIC, "DocumentTitle,RIC"),
        (["RIC", "IssueISIN"], "DocumentTitle,RIC,IssueISIN"),
        ([SymbolTypes.RIC, SymbolTypes.ISIN], "DocumentTitle,RIC,IssueISIN"),
        ("", "DocumentTitle"),
        ([], "DocumentTitle"),
        (None, "DocumentTitle"),
    ],
)
def test_to_symbol_types(input_value, expected_value):
    definition = symbol_conversion.Definition("", to_symbol_types=input_value)

    assert definition.to_symbol_types == expected_value


def test_to_symbol_types_default_value():
    definition = symbol_conversion.Definition("")

    assert definition.to_symbol_types == ",".join(
        ["DocumentTitle"] + list(SYMBOL_TYPE_VALUES)
    )

    assert (
        definition.to_symbol_types
        == "DocumentTitle,RIC,IssueISIN,CUSIP,SEDOL,TickerSymbol,IssuerOAPermID,FundClassLipperID"
    )


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        ("ABW", "RCSExchangeCountry eq 'G:AD'"),
        (CountryCode.ABW, "RCSExchangeCountry eq 'G:AD'"),
        ("G:AD", "RCSExchangeCountry eq 'G:AD'"),
        ("", None),
        ([], None),
        (None, None),
    ],
)
def test_preferred_country_code(input_value, expected_value):
    definition = symbol_conversion.Definition("", preferred_country_code=input_value)

    assert definition.preferred_country_code == expected_value


def test_preferred_country_code_default_value():
    definition = symbol_conversion.Definition("")

    assert definition.preferred_country_code == None


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        ("Commodities", [AssetClass.COMMODITIES]),
        ("COMMODITIES", [AssetClass.COMMODITIES]),
        (AssetClass.COMMODITIES, [AssetClass.COMMODITIES]),
        ("", None),
        ([], None),
        (None, None),
    ],
)
def test_asset_class(input_value, expected_value):
    definition = symbol_conversion.Definition("", asset_class=input_value)

    assert definition.asset_class == expected_value


def test_asset_class_default_value():
    definition = symbol_conversion.Definition("")

    assert definition.asset_class == None


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        ("Active", AssetState.ACTIVE),
        ("ACTIVE", AssetState.ACTIVE),
        (AssetState.ACTIVE, AssetState.ACTIVE),
        ("", None),
        ([], None),
        (None, None),
    ],
)
def test_asset_state(input_value, expected_value):
    definition = symbol_conversion.Definition("", asset_state=input_value)

    assert definition.asset_state == expected_value


def test_asset_state_default_value():
    definition = symbol_conversion.Definition("")

    assert definition.asset_state == None


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        ({"ABW"}, {"ABW"}),
        ("G:AD", "G:AD"),
        ("", None),
        ([], None),
        (None, None),
    ],
)
def test_extended_params(input_value, expected_value):
    definition = symbol_conversion.Definition("", extended_params=input_value)

    assert definition.extended_params == expected_value


def test_extended_params_default_value():
    definition = symbol_conversion.Definition("")

    assert definition.extended_params == None


@pytest.mark.parametrize(
    "input_asset_state, input_asset_class, expected_value",
    [
        (
            AssetState.ACTIVE,
            AssetClass.COMMODITIES,
            "AssetState eq 'AC' and (SearchAllCategoryv3 in ('Commodities'))",
        ),
        (
            AssetState.INACTIVE,
            AssetClass.EQUITIES,
            "(AssetState ne 'AC' and AssetState ne null) and (SearchAllCategoryv3 in ('Equities'))",
        ),
        (
            AssetState.ACTIVE,
            [AssetClass.COMMODITIES, AssetClass.EQUITIES],
            "AssetState eq 'AC' and (SearchAllCategoryv3 in ('Commodities' 'Equities'))",
        ),
        (
            AssetState.INACTIVE,
            [AssetClass.EQUITIES, AssetClass.EQUITIES],
            "(AssetState ne 'AC' and AssetState ne null) and (SearchAllCategoryv3 in ('Equities' 'Equities'))",
        ),
    ],
)
def test_filter(input_asset_state, input_asset_class, expected_value):
    definition = symbol_conversion.Definition(
        "", asset_state=input_asset_state, asset_class=input_asset_class
    )

    assert definition._filter == expected_value


def test_filter_with_asset_state():
    definition = symbol_conversion.Definition("", asset_state=AssetState.ACTIVE)

    assert definition._filter == "AssetState eq 'AC'"


def test_filter_with_asset_class():
    definition = symbol_conversion.Definition(
        "", asset_class=[AssetClass.EQUITIES, AssetClass.EQUITIES]
    )

    assert (
        definition._filter
        == "AssetState eq 'AC' and (SearchAllCategoryv3 in ('Equities' 'Equities'))"
    )


def test_filter_default_value():
    definition = symbol_conversion.Definition("")

    assert definition._filter == None



from collections import namedtuple

from ...conftest import load_json

fund_and_ref_response = load_json(
    "./tests/unit/content/fundamental/fund_and_ref_response.json"
)

ExpectedArgs = namedtuple("ExpectedArgs", "universe, fields")

PATH_MOCK_FUNC = "refinitiv.data.content.fundamental_and_reference._data_provider.validate_correct_format_parameters"


def side_effect_func(*_, **kwargs):
    return kwargs



__all__ = (
    "add_periods",
    "count_periods",
    "date_schedule",
    "is_working_day",
    "holidays",
    "PeriodType",
    "DayCountBasis",
    "DateMovingConvention",
    "EndOfMonthConvention",
    "DateScheduleFrequency",
    "DayOfWeek",
)

from ._add_periods import add_periods
from ._count_periods import count_periods
from ._date_schedule import date_schedule
from ._is_working_day import is_working_day
from ._holidays import holidays

from ...content.ipa._enums import (
    PeriodType,
    DayCountBasis,
    DateMovingConvention,
    EndOfMonthConvention,
    DateScheduleFrequency,
    DayOfWeek,
)



from refinitiv.data._tools.templates import InvalidPlaceholderError


class TestInvalidPlaceholderError:
    def test_noncut(self):
        exc = InvalidPlaceholderError(2, "abcd")
        assert str(exc) == (
            "Invalid placeholder in the template string: line 1, col 3:\n"
            "abcd\n"
            "--^"
        )

    def test_noncut_multiline(self):
        exc = InvalidPlaceholderError(7, "abcd\nefgh")
        assert str(exc) == (
            "Invalid placeholder in the template string: line 2, col 3:\n"
            "efgh\n"
            "--^"
        )

    def test_cursor_inside_limit(self):
        exc = InvalidPlaceholderError(4, "one two three", 7)
        assert str(exc) == (
            "Invalid placeholder in the template string: line 1, col 5:\n"
            "one two\n"
            "----^"
        )

    def test_cursor_outside_limit(self):
        exc = InvalidPlaceholderError(9, "one two three", 7)
        assert str(exc) == (
            "Invalid placeholder in the template string: line 1, col 10:\n"
            " two th\n"
            "------^"
        )

    def test_cursor_inside_limit_with_padding(self):
        exc = InvalidPlaceholderError(4, "one two three", 7, 3)
        assert str(exc) == (
            "Invalid placeholder in the template string: line 1, col 5:\n"
            "ne two \n"
            "---^"
        )



from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions import (
    CrossCurrencyInstrumentsSegment,
)
from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions import (
    CrossCurrencyConstituentsDescription,
    CrossCurrencyCurveParameters,
)


def test_cross_currency_instruments_segment():
    # given
    expected_dict = {"startDate": "string", "constituents": {}, "curveParameters": {}}

    # when
    testing_obj = CrossCurrencyInstrumentsSegment(
        start_date="string",
        constituents=CrossCurrencyConstituentsDescription(),
        curve_parameters=CrossCurrencyCurveParameters(),
    )

    # then
    assert testing_obj.get_dict() == expected_dict

    assert testing_obj.start_date == "string"
    assert testing_obj.constituents == CrossCurrencyConstituentsDescription()
    assert testing_obj.curve_parameters == CrossCurrencyCurveParameters()



__version__ = "1.0.129"

from ._bid_ask_fields_description import *
from ._bid_ask_fields_description import *
from ._constituents_description import *
from ._curve_parameters import *
from ._instrument_definition import *
from ._constituents_description import *
from ._instruments_segment import *
from ._field_description import *
from ._field_formula_description import *
from ._formula_parameter_description import *
from ._fx_forward_instrument_definition import *
from ._fx_forward_instrument_description import *
from ._fx_forward_turn_fields import *
from ._fx_spot_instrument_definition import *
from ._fx_spot_instrument_description import *
from ._override_bid_ask import *
from ._override_bid_ask_fields import *
from ._override_fx_forward_turn import *
from ._turn_adjustment import *



from ._package_manager import create_manager
from ...._tools._common import cached_property


class PackageManager:
    def __init__(self, package_name, session=None) -> None:
        self.package_name = package_name
        self.session = session

    @cached_property
    def _manager(self):
        return create_manager(self.package_name, self.session)

    def update_files(self) -> None:
        """Download the latest update files and the latest init file if none has been downloaded yet."""
        self._manager.update_files()

    def reset_files(self) -> None:
        """Cleans up files and force the download of the latest init update files and all subsequent update files.
        Note: A call to reset_files() is equivalent to a call to cleanup_files() followed by update_files()."""
        self._manager.reset_files()

    def cleanup_files(self) -> None:
        """Remove all previously downloaded files."""
        self._manager.cleanup_files()

    def cleanup_db(self) -> None:
        """Clean up the db accordingly to config file."""
        self._manager.cleanup_db()

    def update_db(self) -> None:
        """Update database
        Note: call cleanup_db() beforehand if there is a new init file."""
        self._manager.update_db()



# coding: utf8

from typing import Optional

from .._base import BinaryDefinition
from .._enums import (
    FxBinaryType,
)


class FxBinaryDefinition(BinaryDefinition):
    """
    Parameters
    ----------
    binary_type : FxBinaryType, optional
        The type of a binary option. the possible values are:
        - onetouchimmediate: the option expires in-the-money if the trigger is reached
          at any time prior to option expiration.the option premium is paid immediately.
        - onetouchdeferred: the option expires in-the-money if the trigger is reached at
          any time prior to option expiration. the option premium payment is deferred.
        - notouch: the option expires in-the-money if the trigger is not reached prior
          to expiration.
        - digital: the option expires in-the-money if the trigger is reached at the
          option expiry date. mandatory for binary options.
    payout_amount : float, optional
        The payout amount of the option. the default value is '1,000,000'.
    payout_ccy : str, optional
        The trade currency, which is either a domestic or foreign currency. either
        payoutccy or settlementtype can be used at a time. payoutccy="foreign currency"
        is equivalent to settlementtype ="physical", and payoutccy="domestic currency"
        is equivalent to settlementtype ="cash". the value is expressed in iso 4217
        alphabetical format (e.g. 'usd').
    trigger : float, optional
        The trigger of the binary option.
    """

    def __init__(
        self,
        binary_type: Optional[FxBinaryType] = None,
        payout_amount: Optional[float] = None,
        payout_ccy: Optional[str] = None,
        trigger: Optional[float] = None,
    ) -> None:
        super().__init__()
        self.binary_type = binary_type
        self.payout_amount = payout_amount
        self.payout_ccy = payout_ccy
        self.trigger = trigger

    @property
    def binary_type(self):
        """
        Binary Type of the digital option
        :return: enum FxBinaryType
        """
        return self._get_enum_parameter(FxBinaryType, "binaryType")

    @binary_type.setter
    def binary_type(self, value):
        self._set_enum_parameter(FxBinaryType, "binaryType", value)

    @property
    def payout_amount(self):
        """
        Payout of the binary option. Default
        :return: float
        """
        return self._get_parameter("payoutAmount")

    @payout_amount.setter
    def payout_amount(self, value):
        self._set_parameter("payoutAmount", value)

    @property
    def payout_ccy(self):
        """
        Payout Currency of the binary option. Default
        :return: str
        """
        return self._get_parameter("payoutCcy")

    @payout_ccy.setter
    def payout_ccy(self, value):
        self._set_parameter("payoutCcy", value)

    @property
    def trigger(self):
        """
        trigger of the binary option.
        :return: float
        """
        return self._get_parameter("trigger")

    @trigger.setter
    def trigger(self, value):
        self._set_parameter("trigger", value)



from tests.unit.conftest import StubResponse

HP_EVENTS = StubResponse(
    [
        {
            "universe": {"ric": "EUR="},
            "adjustments": ["exchangeCorrection", "manualCorrection"],
            "defaultPricingField": "BID",
            "headers": [
                {"name": "DATE_TIME", "type": "string"},
                {"name": "EVENT_TYPE", "type": "string"},
                {"name": "RTL", "type": "number", "decimalChar": "."},
                {"name": "BID", "type": "number", "decimalChar": "."},
                {"name": "ASK", "type": "number", "decimalChar": "."},
                {"name": "MID_PRICE", "type": "number", "decimalChar": "."},
                {"name": "DSPLY_NAME", "type": "string"},
                {"name": "SRC_REF1", "type": "string"},
                {"name": "DLG_CODE1", "type": "string"},
                {"name": "CTBTR_1", "type": "string"},
                {"name": "CTB_LOC1", "type": "string"},
                {"name": "QUALIFIERS", "type": "string"},
            ],
            "data": [],
            "status": {
                "code": "TS.Intraday.Warning.95004",
                "message": "Trades interleaving with corrections is currently not supported. Corrections will not be returned.",
            },
        }
    ]
)

CUSTOM_INSTRUMENTS_EVENTS = StubResponse(
    [
        {
            "universe": {"ric": "S)Batman_9ccfafd6.ertvd-111923"},
            "adjustments": ["exchangeCorrection", "manualCorrection"],
            "defaultPricingField": "BID",
            "headers": [
                {"name": "DATE_TIME", "type": "string"},
                {"name": "EVENT_TYPE", "type": "string"},
                {"name": "RTL", "type": "number", "decimalChar": "."},
                {"name": "BID", "type": "number", "decimalChar": "."},
                {"name": "ASK", "type": "number", "decimalChar": "."},
                {"name": "MID_PRICE", "type": "number", "decimalChar": "."},
                {"name": "DSPLY_NAME", "type": "string"},
                {"name": "SRC_REF1", "type": "string"},
                {"name": "DLG_CODE1", "type": "string"},
                {"name": "CTBTR_1", "type": "string"},
                {"name": "CTB_LOC1", "type": "string"},
                {"name": "QUALIFIERS", "type": "string"},
            ],
            "data": [],
            "status": {
                "code": "TS.Intraday.Warning.95004",
                "message": "Trades interleaving with corrections is currently not supported. Corrections will not be returned.",
            },
        }
    ]
)

OWNERSHIP_ORG_INFO = StubResponse(
    {
        "links": {"count": 1},
        "variability": "fixed",
        "universe": [
            {
                "Instrument": "TRI.N",
                "Company Common Name": "Thomson Reuters Corp",
                "Organization PermID": "4295861160",
                "Reporting Currency": "USD",
            }
        ],
        "data": [],
        "headers": [
            {
                "name": "instrument",
                "title": "Instrument",
                "type": "string",
                "description": "The requested Instrument as defined by the user.",
            },
            {
                "name": "TR.CommonName",
                "title": "Company Name",
                "type": "string",
                "description": "Where available provides the name of the organization most commonly used. Provides primary share class name for funds.",
            },
            {
                "name": "TR.ExchangeName",
                "title": "ExchangeName",
                "type": "string",
                "description": "Exchange Name",
            },
            {
                "name": "TR.OrgTRBCActivity",
                "title": "TRBC Industry",
                "type": "string",
                "description": "The Refinitiv Business Classification (TRBC) Activity Description. TRBC Classifies companies with increasing granularity by Economic Sector, Business Sector, Industry Group, Industry and Activity.",
            },
            {
                "name": "TR.TRBCActivityCode",
                "title": "TRBC IndustryCode",
                "type": "string",
                "description": "The Refinitiv Business Classification (TRBC) Activity classifies companies at the most granular level and indicates the business activity. This is the fifth level in the classification, which represents the activities of the organizations.",
            },
            {
                "name": "TR.HQCountryCode",
                "title": "Country Code",
                "type": "string",
                "description": "Country Code for Organization Headquarters, ISO 3166 standard. Also known as Country of Domicile.",
            },
            {
                "name": "TR.HeadquartersCountry",
                "title": "Country Name",
                "type": "string",
                "description": "Country of Headquarters, also known as Country of Domicile.",
            },
            {
                "name": "TR.FreeFloat",
                "title": "FreeFloat",
                "type": "number",
                "decimalChar": ".",
                "description": "Shares Outstanding - Treasury Shares (if applicable) -\n(Shares held by Strategic Entities–Corporations +\n  Shares held by Strategic Entities–Holding Companies +\n  Shares held by Strategic Entities–Individuals  +\n  Shares held by Strategic Entities–Government Agencies)",
            },
            {
                "name": "TR.FreeFloatPct",
                "title": "FreeFloat %",
                "type": "number",
                "decimalChar": ".",
                "description": "Free Float as a percentage of traded shares",
            },
            {
                "name": "TR.SharesOutstanding",
                "title": "SharesOutstanding",
                "type": "number",
                "decimalChar": ".",
                "description": "The Outstanding Shares represents the issue level outstanding shares. The outstanding share is the number of shares that have been issued and are currently owned by investors. This may include shares that are publicly traded and not traded, such as restricted shares owned by Issue employees etc. Shares that have been repurchased by the Issue as treasury stock are excluded.",
            },
            {
                "name": "TR.SharesHeldByStrategicInvestors",
                "title": "SharesHeld by StrategicInvestors",
                "type": "number",
                "decimalChar": ".",
                "description": "Represents the number of shares held by strategic investors (Corporations, Holding Companies, Individuals and Government Agencies).",
            },
            {
                "name": "Strategic Entities Ownership%",
                "title": "Strategic Entities Ownership%",
                "type": "number",
                "decimalChar": ".",
                "description": None,
            },
            {
                "name": "Market Capitalization",
                "title": "Market Capitalization",
                "type": "number",
                "decimalChar": ".",
                "description": None,
            },
        ],
    }
)

FUNDAMENTAL_AND_REFERENCE_UDF = StubResponse(
    {
        "responses": [
            {
                "columnHeadersCount": 1,
                "data": [],
                "headerOrientation": "horizontal",
                "headers": [
                    [
                        {"displayName": "Instrument"},
                        {"displayName": "Volume", "field": "TR.VOLUME"},
                    ]
                ],
                "rowHeadersCount": 1,
                "totalColumnsCount": 2,
                "totalRowsCount": 2,
            }
        ]
    }
)

NEWS_HEADLINES_UDF = StubResponse({"headlines": [], "newer": "/headlines?payload=eyJ="})

FUNDAMENTAL_AND_REFERENCE_RDP = StubResponse(
    {
        "links": {"count": 1},
        "variability": "",
        "universe": [
            {
                "Instrument": "IBM",
                "Company Common Name": "International Business Machines Corp",
                "Organization PermID": "4295904307",
                "Reporting Currency": "USD",
            }
        ],
        "data": [],
        "messages": {
            "codes": [[-1, -1]],
            "descriptions": [{"code": -1, "description": "ok"}],
        },
        "headers": [
            {
                "name": "instrument",
                "title": "Instrument",
                "type": "string",
                "description": "The requested Instrument as defined by the user.",
            },
            {
                "name": "TR.Volume",
                "title": "Volume",
                "type": "number",
                "decimalChar": ".",
                "description": "Volume for the latest trading day.  For stock exchanges that trade share by share, Volume is number of shares that traded on the trade date. For stock exchanges that trade in lots, Volume is divided by Lotsize, if the Lotsize is greater than one.",
            },
        ],
    }
)

CAP_FLOOR = StubResponse(
    {
        "headers": [
            {"type": "String", "name": "InstrumentTag"},
            {"type": "String", "name": "InstrumentDescription"},
            {"type": "DateTime", "name": "StartDate"},
            {"type": "DateTime", "name": "EndDate"},
            {"type": "String", "name": "InterestPaymentFrequency"},
            {"type": "String", "name": "IndexRic"},
            {"type": "Float", "name": "CapStrikePercent"},
            {"type": "Float", "name": "FloorStrikePercent"},
            {"type": "String", "name": "NotionalCcy"},
            {"type": "Float", "name": "NotionalAmount"},
            {"type": "Float", "name": "PremiumBp"},
            {"type": "Float", "name": "PremiumPercent"},
            {"type": "Float", "name": "MarketValueInDealCcy"},
            {"type": "Float", "name": "MarketValueInReportCcy"},
            {"type": "String", "name": "ErrorMessage"},
        ],
        "data": [],
    }
)



__all__ = (
    "BuySell",
    "DayCountBasis",
    "Definition",
    "PricingParameters",
    "RepoCurveType",
    "RepoParameters",
    "UnderlyingContract",
    "UnderlyingPricingParameters",
)

from ._definition import Definition
from ._enums import (
    BuySell,
    DayCountBasis,
    RepoCurveType,
)
from ._repo_parameters import RepoParameters
from ._repo_pricing_parameters import PricingParameters
from ._repo_underlying_contract import UnderlyingContract
from ._repo_underlying_pricing_parameters import UnderlyingPricingParameters



from tests.unit.conftest import StubResponse

DF_WITHOUT_NAN_RESPONSES = [
    StubResponse(
        [
            {
                "universe": {"ric": "AMD.O"},
                "interval": "PT1M",
                "summaryTimestampLabel": "startPeriod",
                "adjustments": [
                    "exchangeCorrection",
                    "manualCorrection",
                    "CCH",
                    "CRE",
                    "RTS",
                    "RPO",
                ],
                "defaultPricingField": "TRDPRC_1",
                "qos": {"timeliness": "delayed"},
                "headers": [
                    {"name": "DATE_TIME", "type": "string"},
                    {"name": "HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_PRC", "type": "number", "decimalChar": "."},
                    {"name": "TRDPRC_1", "type": "number", "decimalChar": "."},
                    {"name": "NUM_MOVES", "type": "number", "decimalChar": "."},
                    {"name": "ACVOL_UNS", "type": "number", "decimalChar": "."},
                ],
                "data": [
                    [
                        "2022-05-25T20:01:00.000000000Z",
                        None,
                        None,
                        None,
                        None,
                        None,
                        56358,
                    ],
                    [
                        "2022-05-25T20:00:00.000000000Z",
                        92.7,
                        92.65,
                        92.7,
                        92.65,
                        2,
                        1750467,
                    ],
                    [
                        "2022-05-25T19:59:00.000000000Z",
                        92.71,
                        92.484,
                        92.55,
                        92.7099,
                        2483,
                        619187,
                    ],
                    [
                        "2022-05-25T19:58:00.000000000Z",
                        92.5714,
                        92.4722,
                        92.53,
                        92.5575,
                        1451,
                        339267,
                    ],
                    [
                        "2022-05-25T19:57:00.000000000Z",
                        92.575,
                        92.44,
                        92.55,
                        92.54,
                        1186,
                        306126,
                    ],
                ],
            }
        ]
    ),
    StubResponse(
        [
            {
                "universe": {"ric": "IMNM.O"},
                "interval": "PT1M",
                "summaryTimestampLabel": "startPeriod",
                "adjustments": [
                    "exchangeCorrection",
                    "manualCorrection",
                    "CCH",
                    "CRE",
                    "RTS",
                    "RPO",
                ],
                "defaultPricingField": "TRDPRC_1",
                "qos": {"timeliness": "delayed"},
                "headers": [
                    {"name": "DATE_TIME", "type": "string"},
                    {"name": "HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_PRC", "type": "number", "decimalChar": "."},
                    {"name": "TRDPRC_1", "type": "number", "decimalChar": "."},
                    {"name": "NUM_MOVES", "type": "number", "decimalChar": "."},
                    {"name": "ACVOL_UNS", "type": "number", "decimalChar": "."},
                ],
                "data": [
                    [
                        "2022-05-25T19:59:00.000000000Z",
                        None,
                        None,
                        None,
                        None,
                        None,
                        231,
                    ],
                    [
                        "2022-05-25T19:58:00.000000000Z",
                        None,
                        None,
                        None,
                        None,
                        None,
                        100,
                    ],
                    ["2022-05-25T19:57:00.000000000Z", 3, 3, 3, 3, 1, 100],
                    ["2022-05-25T19:56:00.000000000Z", 3, 3, 3, 3, 2, 224],
                    ["2022-05-25T19:54:00.000000000Z", 3, 3, 3, 3, 1, 104],
                ],
            }
        ]
    ),
]

DF_REQUIREMENT_TWO_INSTS_ONE_FIELD = [
    StubResponse(
        [
            {
                "universe": {"ric": "GBP="},
                "interval": "P1D",
                "summaryTimestampLabel": "endPeriod",
                "adjustments": [
                    "exchangeCorrection",
                    "manualCorrection",
                    "CCH",
                    "CRE",
                    "RTS",
                    "RPO",
                ],
                "defaultPricingField": "BID",
                "headers": [
                    {"name": "DATE", "type": "string"},
                    {"name": "BID", "type": "number", "decimalChar": "."},
                ],
                "data": [
                    ["2022-06-22", 1.2266],
                    ["2022-06-21", 1.2272],
                    ["2022-06-20", 1.225],
                    ["2022-06-17", 1.2224],
                    ["2022-06-16", 1.2351],
                    ["2022-06-15", 1.2178],
                    ["2022-06-14", 1.1993],
                    ["2022-06-13", 1.2134],
                    ["2022-06-10", 1.2314],
                    ["2022-06-09", 1.249],
                ],
                "meta": {
                    "blendingEntry": {
                        "headers": [
                            {"name": "DATE", "type": "string"},
                            {"name": "BID", "type": "number", "decimalChar": "."},
                        ],
                        "data": [["2022-06-23", None]],
                    }
                },
            }
        ]
    ),
    StubResponse(
        [
            {
                "universe": {"ric": "G234BP="},
                "status": {
                    "code": "TS.Interday.UserRequestError.70005",
                    "message": "The universe is not found.",
                },
            }
        ]
    ),
]

DF_REQUIREMENT_TWO_INSTS_ONE_IS_BAD_WITHOUT_FIELDS = [
    StubResponse(
        [
            {
                "universe": {"ric": "GBP="},
                "adjustments": ["exchangeCorrection", "manualCorrection"],
                "defaultPricingField": "BID",
                "headers": [
                    {"name": "DATE_TIME", "type": "string"},
                    {"name": "EVENT_TYPE", "type": "string"},
                    {"name": "RTL", "type": "number", "decimalChar": "."},
                    {"name": "BID", "type": "number", "decimalChar": "."},
                    {"name": "ASK", "type": "number", "decimalChar": "."},
                    {"name": "MID_PRICE", "type": "number", "decimalChar": "."},
                    {"name": "DSPLY_NAME", "type": "string"},
                    {"name": "SRC_REF1", "type": "string"},
                    {"name": "DLG_CODE1", "type": "string"},
                    {"name": "CTBTR_1", "type": "string"},
                    {"name": "CTB_LOC1", "type": "string"},
                    {"name": "QUALIFIERS", "type": "string"},
                ],
                "data": [
                    [
                        "2022-06-23T11:54:12.757000000Z",
                        "quote",
                        8590,
                        1.2219,
                        1.2224,
                        1.22215,
                        None,
                        "NBJX",
                        "NBJJ",
                        "NEDBANK LTD",
                        "JHB",
                        None,
                    ],
                    [
                        "2022-06-23T11:54:12.619000000Z",
                        "quote",
                        8526,
                        1.222,
                        1.2225,
                        1.22225,
                        None,
                        "RAB1",
                        "RABX",
                        "RABOBANKGFM",
                        "LON",
                        None,
                    ],
                    [
                        "2022-06-23T11:54:12.169000000Z",
                        "quote",
                        8462,
                        1.2219,
                        1.2223,
                        1.2221,
                        None,
                        "BCFX",
                        None,
                        "BARCLAYS",
                        "LON",
                        None,
                    ],
                ],
                "status": {
                    "code": "TS.Intraday.Warning.95004",
                    "message": "Trades interleaving with corrections is currently not supported. Corrections will not be returned.",
                },
                "meta": {
                    "blendingEntry": {
                        "headers": [
                            {"name": "COLLECT_DATETIME", "type": "string"},
                            {"name": "RTL", "type": "number", "decimalChar": "."},
                        ],
                        "data": [["2022-06-23T11:54:12.757000000Z", 8590]],
                    }
                },
            }
        ]
    ),
    StubResponse(
        [
            {
                "universe": {"ric": "G234BP="},
                "status": {
                    "code": "TS.Intraday.UserRequestError.90001",
                    "message": "The universe is not found.",
                },
            }
        ]
    ),
]


DF_WITH_EXTENDED_PARAMS = [
    StubResponse(
        [
            {
                "universe": {"ric": "VOD.L"},
                "adjustments": ["exchangeCorrection", "manualCorrection"],
                "defaultPricingField": "TRDPRC_1",
                "headers": [
                    {"name": "DATE_TIME", "type": "string"},
                    {"name": "EVENT_TYPE", "type": "string"},
                    {"name": "RTL", "type": "number", "decimalChar": "."},
                    {"name": "SEQNUM", "type": "string"},
                    {"name": "BID", "type": "number", "decimalChar": "."},
                    {"name": "BIDSIZE", "type": "number", "decimalChar": "."},
                    {"name": "ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASKSIZE", "type": "number", "decimalChar": "."},
                    {"name": "MID_PRICE", "type": "number", "decimalChar": "."},
                    {"name": "IMB_SH", "type": "number", "decimalChar": "."},
                    {"name": "IMB_SIDE", "type": "string", "isEnum": True},
                    {"name": "QUALIFIERS", "type": "string"},
                ],
                "data": [
                    [
                        "2022-07-19T15:52:52.900000000Z",
                        "quote",
                        50624,
                        "9076320",
                        130.82,
                        12365,
                        131,
                        2153541,
                        130.91,
                        None,
                        None,
                        "[BID_TONE]",
                    ],
                    [
                        "2022-07-19T15:40:07.641000000Z",
                        "quote",
                        50544,
                        "9075948",
                        130.94,
                        80476,
                        131,
                        2153541,
                        130.97,
                        None,
                        None,
                        "[ASK_TONE]",
                    ],
                    [
                        "2022-07-19T15:40:00.112000000Z",
                        "quote",
                        50528,
                        "9075850",
                        130.94,
                        80476,
                        131,
                        2203541,
                        130.97,
                        None,
                        None,
                        "[ASK_TONE]",
                    ],
                    [
                        "2022-07-19T15:39:38.390000000Z",
                        "quote",
                        50496,
                        "9075694",
                        130.94,
                        80476,
                        131,
                        2503541,
                        130.97,
                        None,
                        None,
                        "[ASK_TONE]",
                    ],
                    [
                        "2022-07-19T15:39:01.320000000Z",
                        "quote",
                        50480,
                        "9075524",
                        130.94,
                        80476,
                        131,
                        2453541,
                        130.97,
                        None,
                        None,
                        "[ASK_TONE]",
                    ],
                ],
            },
        ]
    ),
    StubResponse(
        [
            {
                "universe": {"ric": "GOOG.O"},
                "adjustments": ["exchangeCorrection", "manualCorrection"],
                "defaultPricingField": "TRDPRC_1",
                "headers": [
                    {"name": "DATE_TIME", "type": "string"},
                    {"name": "EVENT_TYPE", "type": "string"},
                    {"name": "RTL", "type": "number", "decimalChar": "."},
                    {"name": "BID", "type": "number", "decimalChar": "."},
                    {"name": "BIDSIZE", "type": "number", "decimalChar": "."},
                    {"name": "BID_MMID1", "type": "string"},
                    {"name": "ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASKSIZE", "type": "number", "decimalChar": "."},
                    {"name": "ASK_MMID1", "type": "string"},
                    {"name": "MID_PRICE", "type": "number", "decimalChar": "."},
                    {"name": "UPLIMIT", "type": "number", "decimalChar": "."},
                    {"name": "LOLIMIT", "type": "number", "decimalChar": "."},
                    {"name": "LIMIT_INDQ", "type": "string", "isEnum": True},
                    {"name": "SH_SAL_RES", "type": "string", "isEnum": True},
                    {"name": "QUALIFIERS", "type": "string"},
                ],
                "data": [
                    [
                        "2022-07-19T15:59:58.425000000Z",
                        "quote",
                        60400,
                        113.33,
                        200,
                        "DEX",
                        113.35,
                        100,
                        "NAS",
                        None,
                        118.44,
                        107.16,
                        "BOE",
                        "N",
                        "   [PRC_QL_CD];   [PRC_QL3];A[GV1_TEXT]",
                    ],
                    [
                        "2022-07-19T15:59:58.424000000Z",
                        "quote",
                        60352,
                        113.33,
                        200,
                        "DEX",
                        113.35,
                        100,
                        "MMX",
                        None,
                        118.44,
                        107.16,
                        "BOE",
                        "N",
                        "   [PRC_QL_CD];   [PRC_QL3];A[GV1_TEXT]",
                    ],
                    [
                        "2022-07-19T15:59:58.410000000Z",
                        "quote",
                        60336,
                        113.33,
                        200,
                        "DEX",
                        113.35,
                        200,
                        "NAS",
                        None,
                        118.44,
                        107.16,
                        "BOE",
                        "N",
                        "   [PRC_QL_CD];   [PRC_QL3];A[GV1_TEXT]",
                    ],
                    [
                        "2022-07-19T15:59:58.409000000Z",
                        "quote",
                        60320,
                        113.33,
                        200,
                        "DEX",
                        113.35,
                        100,
                        "MMX",
                        None,
                        118.44,
                        107.16,
                        "BOE",
                        "N",
                        "   [PRC_QL_CD];   [PRC_QL3];A[GV1_TEXT]",
                    ],
                    [
                        "2022-07-19T15:59:58.409000000Z",
                        "quote",
                        60272,
                        113.33,
                        200,
                        "DEX",
                        113.36,
                        200,
                        "NAS",
                        None,
                        118.44,
                        107.16,
                        "BOE",
                        "N",
                        "   [PRC_QL_CD];   [PRC_QL3];A[GV1_TEXT]",
                    ],
                ],
            }
        ]
    ),
]

DF_EMPTY_ONE_INST = [
    StubResponse(
        [
            {
                "universe": {"ric": "LSEG.L"},
                "interval": "PT1M",
                "summaryTimestampLabel": "startPeriod",
                "adjustments": [
                    "exchangeCorrection",
                    "manualCorrection",
                    "CCH",
                    "CRE",
                    "RTS",
                    "RPO",
                ],
                "defaultPricingField": "TRDPRC_1",
                "headers": [
                    {"name": "DATE_TIME", "type": "string"},
                    {"name": "HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_PRC", "type": "number", "decimalChar": "."},
                    {"name": "TRDPRC_1", "type": "number", "decimalChar": "."},
                    {"name": "NUM_MOVES", "type": "number", "decimalChar": "."},
                    {"name": "ACVOL_UNS", "type": "number", "decimalChar": "."},
                    {"name": "HIGH_YLD", "type": "number", "decimalChar": "."},
                    {"name": "LOW_YLD", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_YLD", "type": "number", "decimalChar": "."},
                    {"name": "YIELD", "type": "number", "decimalChar": "."},
                    {"name": "BID_HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "BID_LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_BID", "type": "number", "decimalChar": "."},
                    {"name": "BID", "type": "number", "decimalChar": "."},
                    {"name": "BID_NUMMOV", "type": "number", "decimalChar": "."},
                    {"name": "ASK_HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "ASK_LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASK_NUMMOV", "type": "number", "decimalChar": "."},
                    {"name": "MID_HIGH", "type": "number", "decimalChar": "."},
                    {"name": "MID_LOW", "type": "number", "decimalChar": "."},
                    {"name": "MID_OPEN", "type": "number", "decimalChar": "."},
                    {"name": "MID_PRICE", "type": "number", "decimalChar": "."},
                ],
                "data": [],
            }
        ]
    )
]

DF_EMPTY_TWO_INSTS = [
    StubResponse(
        [
            {
                "universe": {"ric": "LSEG.L"},
                "interval": "PT1M",
                "summaryTimestampLabel": "startPeriod",
                "adjustments": [
                    "exchangeCorrection",
                    "manualCorrection",
                    "CCH",
                    "CRE",
                    "RTS",
                    "RPO",
                ],
                "defaultPricingField": "TRDPRC_1",
                "headers": [
                    {"name": "DATE_TIME", "type": "string"},
                    {"name": "HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_PRC", "type": "number", "decimalChar": "."},
                    {"name": "TRDPRC_1", "type": "number", "decimalChar": "."},
                    {"name": "NUM_MOVES", "type": "number", "decimalChar": "."},
                    {"name": "ACVOL_UNS", "type": "number", "decimalChar": "."},
                    {"name": "HIGH_YLD", "type": "number", "decimalChar": "."},
                    {"name": "LOW_YLD", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_YLD", "type": "number", "decimalChar": "."},
                    {"name": "YIELD", "type": "number", "decimalChar": "."},
                    {"name": "BID_HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "BID_LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_BID", "type": "number", "decimalChar": "."},
                    {"name": "BID", "type": "number", "decimalChar": "."},
                    {"name": "BID_NUMMOV", "type": "number", "decimalChar": "."},
                    {"name": "ASK_HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "ASK_LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASK_NUMMOV", "type": "number", "decimalChar": "."},
                    {"name": "MID_HIGH", "type": "number", "decimalChar": "."},
                    {"name": "MID_LOW", "type": "number", "decimalChar": "."},
                    {"name": "MID_OPEN", "type": "number", "decimalChar": "."},
                    {"name": "MID_PRICE", "type": "number", "decimalChar": "."},
                ],
                "data": [],
            }
        ]
    ),
    StubResponse(
        [
            {
                "universe": {"ric": "VOD.L"},
                "interval": "PT1M",
                "summaryTimestampLabel": "startPeriod",
                "adjustments": [
                    "exchangeCorrection",
                    "manualCorrection",
                    "CCH",
                    "CRE",
                    "RTS",
                    "RPO",
                ],
                "defaultPricingField": "TRDPRC_1",
                "headers": [
                    {"name": "DATE_TIME", "type": "string"},
                    {"name": "HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_PRC", "type": "number", "decimalChar": "."},
                    {"name": "TRDPRC_1", "type": "number", "decimalChar": "."},
                    {"name": "NUM_MOVES", "type": "number", "decimalChar": "."},
                    {"name": "ACVOL_UNS", "type": "number", "decimalChar": "."},
                    {"name": "HIGH_YLD", "type": "number", "decimalChar": "."},
                    {"name": "LOW_YLD", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_YLD", "type": "number", "decimalChar": "."},
                    {"name": "YIELD", "type": "number", "decimalChar": "."},
                    {"name": "BID_HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "BID_LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_BID", "type": "number", "decimalChar": "."},
                    {"name": "BID", "type": "number", "decimalChar": "."},
                    {"name": "BID_NUMMOV", "type": "number", "decimalChar": "."},
                    {"name": "ASK_HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "ASK_LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASK_NUMMOV", "type": "number", "decimalChar": "."},
                    {"name": "MID_HIGH", "type": "number", "decimalChar": "."},
                    {"name": "MID_LOW", "type": "number", "decimalChar": "."},
                    {"name": "MID_OPEN", "type": "number", "decimalChar": "."},
                    {"name": "MID_PRICE", "type": "number", "decimalChar": "."},
                ],
                "data": [],
            }
        ]
    ),
]

DF_EMPTY_ONE_INST_IS_BAD = [
    StubResponse(
        [
            {
                "universe": {"ric": "LSEG.L"},
                "interval": "PT1M",
                "summaryTimestampLabel": "startPeriod",
                "adjustments": [
                    "exchangeCorrection",
                    "manualCorrection",
                    "CCH",
                    "CRE",
                    "RTS",
                    "RPO",
                ],
                "defaultPricingField": "TRDPRC_1",
                "headers": [
                    {"name": "DATE_TIME", "type": "string"},
                    {"name": "HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_PRC", "type": "number", "decimalChar": "."},
                    {"name": "TRDPRC_1", "type": "number", "decimalChar": "."},
                    {"name": "NUM_MOVES", "type": "number", "decimalChar": "."},
                    {"name": "ACVOL_UNS", "type": "number", "decimalChar": "."},
                    {"name": "HIGH_YLD", "type": "number", "decimalChar": "."},
                    {"name": "LOW_YLD", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_YLD", "type": "number", "decimalChar": "."},
                    {"name": "YIELD", "type": "number", "decimalChar": "."},
                    {"name": "BID_HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "BID_LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_BID", "type": "number", "decimalChar": "."},
                    {"name": "BID", "type": "number", "decimalChar": "."},
                    {"name": "BID_NUMMOV", "type": "number", "decimalChar": "."},
                    {"name": "ASK_HIGH_1", "type": "number", "decimalChar": "."},
                    {"name": "ASK_LOW_1", "type": "number", "decimalChar": "."},
                    {"name": "OPEN_ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASK", "type": "number", "decimalChar": "."},
                    {"name": "ASK_NUMMOV", "type": "number", "decimalChar": "."},
                    {"name": "MID_HIGH", "type": "number", "decimalChar": "."},
                    {"name": "MID_LOW", "type": "number", "decimalChar": "."},
                    {"name": "MID_OPEN", "type": "number", "decimalChar": "."},
                    {"name": "MID_PRICE", "type": "number", "decimalChar": "."},
                ],
                "data": [],
            }
        ]
    ),
    StubResponse(
        [
            {
                "universe": {"ric": "VOD.Lffff"},
                "status": {
                    "code": "TS.Intraday.UserRequestError.90001",
                    "message": "The universe is not found.",
                },
            }
        ]
    ),
]

DF_EMPTY_ONE_INST_IS_BAD_OTHER_WITHOUT_HEADERS = [
    StubResponse(
        [
            {
                "universe": {"ric": "EUR="},
                "adjustments": ["exchangeCorrection", "manualCorrection"],
                "defaultPricingField": "BID",
                "headers": [],
                "data": [],
            }
        ]
    ),
    StubResponse(
        [
            {
                "universe": {"ric": "GBP"},
                "status": {
                    "code": "TS.Intraday.UserRequestError.90001",
                    "message": "The universe is not found.",
                },
            }
        ]
    ),
]

DF_EMPTY_WITHOUT_HEADERS = [
    StubResponse(
        [
            {
                "universe": {"ric": "EUR="},
                "adjustments": ["exchangeCorrection", "manualCorrection"],
                "defaultPricingField": "BID",
                "headers": [],
                "data": [],
            }
        ]
    ),
    StubResponse(
        [
            {
                "universe": {"ric": "GBP="},
                "adjustments": ["exchangeCorrection", "manualCorrection"],
                "defaultPricingField": "BID",
                "headers": [],
                "data": [],
            }
        ]
    ),
]

INST_IS_BAD = [
    StubResponse(
        [
            {
                "universe": {"ric": "VOD.Lffff"},
                "status": {
                    "code": "TS.Intraday.UserRequestError.90001",
                    "message": "The universe is not found.",
                },
            }
        ]
    ),
]

INSTS_ARE_BAD = [
    StubResponse(
        [
            {
                "universe": {"ric": "LSEG.L.ffff"},
                "status": {
                    "code": "TS.Intraday.UserRequestError.90001",
                    "message": "The universe is not found.",
                },
            }
        ]
    ),
    StubResponse(
        [
            {
                "universe": {"ric": "VOD.Lffff"},
                "status": {
                    "code": "TS.Intraday.UserRequestError.90001",
                    "message": "The universe is not found.",
                },
            }
        ]
    ),
]



# coding: utf8

__all__ = ["Axis"]

from enum import Enum, unique


@unique
class Axis(Enum):
    """
    The enumerate that specifies the unit for the axis.
    """

    X = "X"
    Y = "Y"
    Z = "Z"
    DATE = "Date"
    DELTA = "Delta"
    EXPIRY = "Expiry"
    MONEYNESS = "Moneyness"
    STRIKE = "Strike"
    TENOR = "Tenor"



import numpy

import refinitiv.data as rd
from tests.unit.conftest import StubSession


def test_snapping_zero_values_to_pd_na_in_the_pricing_stream():
    # given
    session = StubSession(is_open=True)
    field = "EURO_NETCH"
    universe = "UAH="
    stream = rd.content.pricing.Definition(universe=universe, fields=field).get_stream(
        session
    )
    stream._stream._do_open = lambda *_, **__: None
    # fmt: off
    stream._stream._stream_by_name[universe]._record = {
        'ID': 5, 'Type': 'Refresh', 'Key': {'Service': 'ELEKTRON_DD', 'Name': universe},
        'State': {'Stream': 'Open', 'Data': 'Ok'},
        'Qos': {'Timeliness': 'Realtime', 'Rate': 'JitConflated'},
        'PermData': 'AwEBV2w=', 'SeqNumber': 574, 'Fields': {field: 0.0}
    }
    # fmt: on

    # when
    stream.open()
    df = stream.get_snapshot()

    # then
    assert isinstance(df[field][0], numpy.int64)



__all__ = ("chain", "Definition", "Response", "contribute", "contribute_async")

from ...delivery._data._data_provider import Response
from ._definition import Definition
from . import chain
from ...delivery._stream.contrib import contribute, contribute_async



from unittest.mock import patch

import pytest
from pandas import Timestamp

from refinitiv.data._core.session import set_default
from refinitiv.data._fin_coder_layer.get_data import (
    convert_types,
    get_data,
    update_universe,
)
from refinitiv.data.delivery._data._request import Request
from refinitiv.data.discovery import Peers
from tests.unit.conftest import StubResponse, StubSession, send_ws_messages
from tests.unit.delivery.stream.conftest import StubWebSocketApp
from . import data_for_tests as td

args_names = (
    f"adc_df, pricing_df, data, expected_result,"
    f" adc_stub, pricing_stub, is_datetime_dtype, exception_expected"
)


def test_session_is_close():
    # given
    error_message = "Session is not opened. Can't send any request"

    session = StubSession()
    set_default(session)

    # when
    with pytest.raises(ValueError, match=error_message):
        get_data(...)


@pytest.mark.parametrize(
    ("raw", "_universe", "expected_result"),
    [
        (
            {
                "data": [
                    ["GOOG.O", "USD"],
                    ["GOOG.O", "USD"],
                    ["GOOG.O", "USD"],
                    ["GOOG.O", "USD"],
                    ["VOD.L", "USD"],
                    ["VOD.L", "USD"],
                    ["VOD.L", "USD"],
                    ["VOD.L", "USD"],
                    ["EUR=", None],
                ]
            },
            ["GOOG.O", "VOD.L", "EUR="],
            [
                "GOOG.O",
                "GOOG.O",
                "GOOG.O",
                "GOOG.O",
                "VOD.L",
                "VOD.L",
                "VOD.L",
                "VOD.L",
                "EUR=",
            ],
        )
    ],
)
def test_update_universe(raw, _universe, expected_result):
    # given
    # when
    result = update_universe(raw, _universe)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("column", "column_names", "expected_result"),
    [
        (
            ["GOOG.O", "USD", 297242.64071, 297242.64071, "2022-07-05T00:00:00"],
            ["Instrument", "Currency", "Revenue - Mean", "Revenue - Mean", "Date"],
            [
                "GOOG.O",
                "USD",
                297242.64071,
                297242.64071,
                Timestamp("2022-07-05 00:00:00"),
            ],
        ),
    ],
)
def test_convert_types(column, column_names, expected_result):
    # given
    # when
    result = convert_types(column, column_names)

    # then
    assert result == expected_result


infos = {
    "services": [
        {
            "port": 443,
            "location": ["ap-northeast-1a"],
            "transport": "websocket",
            "provider": "aws",
            "endpoint": "ap-northeast-1-aws-1-sm.optimized-pricing-api.refinitiv.net",
            "dataFormat": ["tr_json2"],
        },
    ]
}


def make_http_request(content_data):
    def _http_request(request: Request):
        url = request.url or ""
        if url.endswith("streaming/pricing/v1/"):
            return StubResponse(content_data=infos)
        return StubResponse(content_data=content_data)

    return _http_request


@pytest.mark.parametrize(
    (
        "params",
        "http_responses",
        "stream_data",
        "expected_result",
        "underlying_platform",
    ),
    [
        (
            td.GET_DATA_PARAMS_1,
            td.GET_DATA_RAW_RESPONSE_1,
            td.GET_DATA_STREAM_MESSAGES_1,
            td.GET_DATA_RESULT_1,
            "rdp",
        ),
        (
            td.GET_DATA_PARAMS_2,
            td.GET_DATA_RAW_RESPONSE_2,
            td.GET_DATA_STREAM_MESSAGES_2,
            td.GET_DATA_RESULT_2,
            "rdp",
        ),
        (
            td.GET_DATA_PARAMS_3,
            td.GET_DATA_RAW_RESPONSE_3,
            td.GET_DATA_STREAM_MESSAGES_3,
            td.GET_DATA_RESULT_3,
            "rdp",
        ),
        (
            td.GET_DATA_PARAMS_4,
            td.GET_DATA_RAW_RESPONSE_4,
            td.GET_DATA_STREAM_MESSAGES_4,
            td.GET_DATA_RESULT_4,
            "rdp",
        ),
        (
            td.GET_DATA_PARAMS_5,
            td.GET_DATA_RAW_RESPONSE_5,
            td.GET_DATA_STREAM_MESSAGES_5,
            td.GET_DATA_RESULT_5,
            "rdp",
        ),
        (
            td.GET_DATA_PARAMS_6,
            td.GET_DATA_RAW_RESPONSE_6,
            td.GET_DATA_STREAM_MESSAGES_6,
            td.GET_DATA_RESULT_6,
            "udf",
        ),
        (
            td.GET_DATA_PARAMS_7,
            td.GET_DATA_RAW_RESPONSE_7,
            td.GET_DATA_STREAM_MESSAGES_7,
            td.GET_DATA_RESULT_7,
            "udf",
        ),
        (
            td.GET_DATA_PARAMS_8,
            td.GET_DATA_RAW_RESPONSE_8,
            td.GET_DATA_STREAM_MESSAGES_8,
            td.GET_DATA_RESULT_8,
            "udf",
        ),
    ],
)
@patch("websocket.WebSocketApp", new=StubWebSocketApp)
def test_get_data(
    http_responses, stream_data, expected_result, params, underlying_platform
):
    session = StubSession(is_open=True, stream_auto_reconnection=True)
    session.config.set_param(
        "apis.data.datagrid.underlying-platform", underlying_platform
    )
    session.http_request = make_http_request(http_responses)
    send_ws_messages(session, stream_data)

    result = get_data(**params)

    assert result.to_string() == expected_result

    session.close()


def test_get_data_session_is_not_opened():
    session = StubSession(is_open=False)
    set_default(session)
    with pytest.raises(ValueError):
        get_data(...)


@patch("websocket.WebSocketApp", new=StubWebSocketApp)
def test_no_request_to_adc_if_universe_expander_and_pricing_fields():
    no_request_to_udc = True

    def check_if_no_adc_request(request: Request):
        nonlocal no_request_to_udc
        url = request.url or ""
        if url.endswith("streaming/pricing/v1/"):
            return StubResponse(content_data=infos)
        if url == "test_get_udf_url_root":
            no_request_to_udc = False
        return StubResponse()

    session = StubSession(is_open=True, stream_auto_reconnection=True)
    session.config.set_param("apis.data.datagrid.underlying-platform", "udf")
    session.http_request = check_if_no_adc_request
    send_ws_messages(session, td.GET_DATA_STREAM_MESSAGES_1)
    set_default(session)
    peers = Peers("VOD.L")
    peers._universe = ["EUR="]

    get_data(universe=peers, fields=["bid", "ask"])
    assert no_request_to_udc

    session.close()


@patch("websocket.WebSocketApp", new=StubWebSocketApp)
def test_get_data_duplicate_date():
    expected_str = (
        "  Instrument  Total Revenue per Share       Date  Total Revenue per Share       Date  Revenue from Business Activities - Total     BID     ASK\n"
        "0      IBM.N                63.395314 2021-12-31                63.395314 2020-12-31                               57350000000    <NA>    <NA>\n"
        "1      IBM.N                     <NA> 2020-12-31                61.544967 2019-12-31                                      <NA>    <NA>    <NA>\n"
        "2      IBM.N                     <NA> 2019-12-31                64.642849 2018-12-31                                      <NA>    <NA>    <NA>\n"
        "3      VOD.L                 1.565839 2022-03-31                 1.565839 2021-03-31                               45580000000  106.36  106.38\n"
        "4      VOD.L                     <NA> 2021-03-31                 1.475895 2020-03-31                                      <NA>  106.36  106.38\n"
        "5      VOD.L                     <NA> 2020-03-31                 1.528584 2019-03-31                                      <NA>  106.36  106.38"
    )

    session = StubSession(is_open=True, stream_auto_reconnection=True)
    session.config.set_param("apis.data.datagrid.underlying-platform", "udf")

    session.http_request = make_http_request(td.GET_DATA_DUPLICATE_DATE_HTTP)
    set_default(session)

    send_ws_messages(session, td.GET_DATA_DUPLICATE_DATE_STREAM)

    result_df = get_data(
        universe=["IBM.N", "VOD.L"],
        fields=[
            "BID",
            "ASK",
            "TR.F.TotRevenue",
            "TR.F.TotRevPerShr",
            "TR.F.TotRevenue(SDate=-1,EDate=-3,Period=FY0,Frq=FY).date",
            "TR.F.TotRevPerShr(SDate=0,EDate=-2,Period=FY0,Frq=FY)",
            "TR.F.TotRevPerShr(SDate=0,EDate=-2,Period=FY0,Frq=FY).date",
        ],
        use_field_names_in_headers=False,
    )
    session.close()
    set_default(None)

    assert result_df.to_string() == expected_str



import json
import os
import shutil

import numpy as np
import pandas as pd

from refinitiv.data._tools import to_datetime
from refinitiv.data.content.news.headlines import SortOrder
from tests.integration.helpers import check_if_dataframe_is_not_none


def check_extended_params_were_sent_in_news_request(response, expected_extended_params):
    params_sent_in_request = None
    if response.request_message.method == "GET":  # rdp
        params_sent_in_request = response.request_message.url.params._dict
    elif response.request_message.method == "POST":  # udf
        content = json.loads(response.request_message.content.decode("utf-8"))
        params_sent_in_request = content["Entity"]["W"]
    for key in expected_extended_params.keys():
        try:
            assert expected_extended_params[key] in params_sent_in_request[key]
        except KeyError:
            raise AssertionError(
                f"Key '{key}' not found in params sent in request: {params_sent_in_request}"
            )


def check_story_title(response, story_title):
    assert (
        response.data.story.title == story_title
    ), f"The story with unexpected title received: {response.data.story.title}"


def check_every_story_title_contains_keyword(response, keyword):
    if response.request_message.method == "GET":  # rdp
        for story in response.data.raw["data"]:
            title = story["newsItem"]["itemMeta"]["title"][0]["$"]
            assert (
                keyword.lower() in title.lower()
            ), f"Keyword {keyword} is not included into title {title}"
    elif response.request_message.method == "POST":  # udf
        dataframe = response.data.df
        check_if_dataframe_is_not_none(dataframe)


def check_count_in_response_equal_to(count, response, session_type, expected_raw_count):
    headlines_sum = 0
    if session_type == "PLATFORM":  # rdp
        assert (
            len(response.data.headlines) == count
        ), f"Number of headlines in response is {len(response.data['headlines'])}, but expected {count}"
    elif session_type == "DESKTOP":  # udf
        for page in response.data.raw:
            headlines_sum += len(page["headlines"])
        assert (
            headlines_sum == expected_raw_count
        ), f"Count property in response {headlines_sum} differs from expected {expected_raw_count}"
        assert (
            len(response.data.headlines) == count
        ), f"Number of headlines in response is {len(response.data.headlines)}, but expected {count}"


def retrieve_dates_from_headlines_response(response):
    dataframe = response
    if not isinstance(response, pd.DataFrame):
        dataframe = response.data.df

    return [to_datetime(date) for date in dataframe.index.array]


def check_data_in_response_sorted_by_order(response, order):
    dates = retrieve_dates_from_headlines_response(response)

    if order == SortOrder.new_to_old:
        sorted_dates_descending = sorted(dates, reverse=True)
        assert (
            dates == sorted_dates_descending
        ), f"Retrieved dates from headlines are not sorted from new to old: {dates}"
    elif order == SortOrder.old_to_new:
        sorted_dates_ascending = sorted(dates, reverse=False)
        assert (
            dates == sorted_dates_ascending
        ), f"Retrieved dates from headlines are not sorted from old to new: {dates}"


def check_list_of_dates_are_in_interval(list_of_dates, date_from, date_to):
    date_to = to_datetime(date_to)
    date_from = to_datetime(date_from)

    for date in list_of_dates:
        assert (
            date.timestamp() >= date_from.timestamp()
        ), f"Found the date {date} which is  earlier than {date_from}"
        assert (
            date.timestamp() <= date_to.timestamp()
        ), f"Found the date {date} which is  later  than expected date_to {date_to}"


def check_dates_range_in_response(response, date_from, date_to):
    dates = retrieve_dates_from_headlines_response(response)
    check_list_of_dates_are_in_interval(dates, date_from, date_to)


def check_file_is_saved(filename: str, path: str = None):
    file_path = "./"
    if path:
        file_path=path
    list_of_files = []
    for root, dirs, files in os.walk(file_path):
        for file in files:
            list_of_files.append(file)

    assert filename in list_of_files, f"File {filename} not found"

    # removing a saved file after test
    if path:
        shutil.rmtree(path, ignore_errors=True)
    else:
        os.remove(filename)

def check_news_headlines_date_for_datetime_type(response):
    random_headline = response.data.headlines[0]
    check_if_news_headline_date_is_datetime64(random_headline)
    if hasattr(random_headline, "related_headlines"):
        random_headline = random_headline.related_headlines[0]
        check_if_news_headline_date_is_datetime64(random_headline)


def check_if_news_headline_date_is_datetime64(headline):
    if headline.first_created:
        assert isinstance(headline.first_created.asm8, np.datetime64)
    if headline.version_created:
        assert isinstance(headline.version_created.asm8, np.datetime64)


from enum import Enum, auto


class APIType(Enum):
    CFS = auto()
    CHAINS = auto()
    CURVES_AND_SURFACES = auto()
    CUSTOM_INSTRUMENTS = auto()
    DATA_GRID = auto()
    DATA_STORE = auto()
    DISCOVERY = auto()
    ESG = auto()
    ESTIMATES = auto()
    FILINGS = auto()
    FINANCIAL_CONTRACTS = auto()
    HISTORICAL_PRICING = auto()
    NEWS = auto()
    OWNERSHIP = auto()
    PRICING = auto()
    STREAMING_BENCHMARK = auto()
    STREAMING_CONTRIB = auto()
    STREAMING_CUSTOM = auto()
    STREAMING_CUSTOM_INSTRUMENTS = auto()
    STREAMING_FINANCIAL_CONTRACTS = auto()
    STREAMING_PRICING = auto()
    STREAMING_TRADING = auto()
    DATES_AND_CALENDARS = auto()



# coding: utf8

from ..._object_definition import ObjectDefinition
from ....._types import OptInt, OptFloat


class Turn(ObjectDefinition):
    """
    Parameters
    ----------
    month : int, optional
        Month of the turn period
    rate_percent : float, optional
        Turn rate expressed in percents
    year : int, optional
        Year of the turn period
    """

    def __init__(
        self,
        month: OptInt = None,
        rate_percent: OptFloat = None,
        year: OptInt = None,
    ) -> None:
        super().__init__()
        self.month = month
        self.rate_percent = rate_percent
        self.year = year

    @property
    def month(self):
        """
        Month of the turn period
        :return: int
        """
        return self._get_parameter("month")

    @month.setter
    def month(self, value):
        self._set_parameter("month", value)

    @property
    def rate_percent(self):
        """
        Turn rate expressed in percents
        :return: float
        """
        return self._get_parameter("ratePercent")

    @rate_percent.setter
    def rate_percent(self, value):
        self._set_parameter("ratePercent", value)

    @property
    def year(self):
        """
        Year of the turn period
        :return: int
        """
        return self._get_parameter("year")

    @year.setter
    def year(self, value):
        self._set_parameter("year", value)



import allure
import pytest

from refinitiv.data.content.esg import standard_scores
from refinitiv.data.errors import RDError
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.ownership.helpers import (
    check_response_dataframe_instrument_contains_expected_universes,
)
from tests.integration.helpers import (
    get_async_response_from_definitions,
    check_response_status,
    check_non_empty_response_data,
    check_dataframe_column_date_for_datetime_type,
)


@allure.suite("Content object - ESG Standard Scores")
@allure.feature("Content object - ESG Standard Scores")
@allure.severity(allure.severity_level.CRITICAL)
class TestESGStandardScores:
    @allure.title("Create ESG Standard Scores definition object with valid params")
    @pytest.mark.parametrize(
        "universe,start,end",
        [("5000002406", 0, -2)],
    )
    @pytest.mark.caseid("C40602075")
    @pytest.mark.smoke
    def test_esg_standard_scores_definition_object_with_valid_params_and_get_data(
        self, universe, start, end, open_platform_session
    ):
        response = standard_scores.Definition(
            universe=universe, start=start, end=end
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_response_dataframe_instrument_contains_expected_universes(
            response, universe
        )
        check_non_empty_response_data(response)
        check_dataframe_column_date_for_datetime_type(response)

    @allure.title(
        "Create ESG Standard Scores definition object with invalid params and get error"
    )
    @pytest.mark.caseid("C40602077")
    @pytest.mark.parametrize(
        "universe,expected_error,start,end",
        [
            (
                "5000002406",
                "Error code 400 | Validation error: validation failure list:\nstart in query should be greater than or equal to -2",
                -3,
                -1,
            )
        ],
    )
    def test_esg_standard_scores_definition_object_with_invalid_params_and_get_error(
        self, universe, expected_error, start, end, open_platform_session
    ):
        with pytest.raises(RDError) as error:
            standard_scores.Definition(
                universe=universe, start=start, end=end
            ).get_data()
        assert str(error.value) == expected_error

    @allure.title(
        "Create ESG Standard Scores definition object with use mix valid and invalid data - asynchronously"
    )
    @pytest.mark.caseid("C40602078")
    @pytest.mark.parametrize(
        "universe,invalid_universe,start,end",
        [(["5000002406", "INVALID"], "INVALID", 0, -2)],
    )
    async def test_esg_standard_scores_definition_object_with_use_mix_valid_and_invalid_data(
        self, universe, invalid_universe, start, end, open_platform_session_async
    ):
        valid_response, invalid_response = await get_async_response_from_definitions(
            standard_scores.Definition(universe=universe, start=start, end=end),
            standard_scores.Definition(universe=invalid_universe, start=start, end=end),
        )
        check_response_status(
            response=valid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=None,
            expected_error_message="Failed to resolve identifiers ['INVALID']",
        )
        check_non_empty_response_data(valid_response)
        check_response_status(
            response=invalid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=HttpStatusCode.FOUR_HUNDRED_TWELVE,
            expected_error_message="Unable to resolve all requested identifiers. Requested items: ['INVALID']",
        )



from unittest.mock import MagicMock
from mock_server.mocks import (
    MockNewsHeadlinesResponse,
    MockNewsStoryResponse,
    MockRequestResponse,
    MockHistoricalPricingResponseError,
)


class MockResponseUniversal(MagicMock):
    expires_in_5_min = 5 * 60

    status_code = 200

    headers = {"content-type": "/json"}

    def json(self):
        return {
            "refresh_token": MagicMock(),
            "access_token": MagicMock(),
            "expires_in": MockAuthResponse.expires_in_5_min,
            "service": MagicMock(),
            "headers": MagicMock(),
            "data": [MagicMock()],
        }


class MockAuthResponse(MagicMock):
    expires_in_5_min = 5 * 60

    status_code = 200

    headers = {"content-type": "/json"}

    def json(self):
        return {
            "refresh_token": MagicMock(),
            "access_token": MagicMock(),
            "expires_in": MockAuthResponse.expires_in_5_min,
            "service": MagicMock(),
        }


class MockSession(MagicMock):
    response = MagicMock()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        pass

    async def send(self, request, **kwargs):
        return MockSession.response



from unittest.mock import Mock

import pytest
import refinitiv.data as rd

from tests.unit.conftest import StubSession, StubResponse


def test_import():
    try:
        from refinitiv.data.content.news import headlines
    except ImportError as e:
        assert False, str(e)


def test_attributes():
    from refinitiv.data.content.news import headlines

    assert hasattr(headlines, "Definition")


def test_get_data_called__check_response():
    from refinitiv.data.content.news import headlines

    # given
    response = StubResponse({"data": {}, "headlines": []})
    session = StubSession(is_open=True, response=response)

    definition_layer = headlines.Definition(...)
    definition_layer._check_response = Mock()

    # when
    definition_layer.get_data(session)

    # then
    definition_layer._check_response.assert_called_once()


@pytest.mark.parametrize(
    "news_definition, expected_repr",
    [
        (
            rd.content.news.headlines.Definition("Apple"),
            "<refinitiv.data.content.news.headlines.Definition object at {0} {{query='Apple'}}>",
        ),
        (
            rd.content.news.story.Definition(
                "urn:newsml:reuters.com:20201026:nPt6BSyBh"
            ),
            "<refinitiv.data.content.news.story.Definition object at {0} {{story_id='urn:newsml:reuters.com:20201026:nPt6BSyBh'}}>",
        ),
    ],
)
def test_esg_content_repr(news_definition, expected_repr):
    # given
    obj_id = hex(id(news_definition))

    # when
    s = repr(news_definition)

    # then
    assert s == expected_repr.format(obj_id), s



import abc
import os
from dataclasses import dataclass
from itertools import compress
from typing import List, TYPE_CHECKING, Union

from .proxy_info import ProxyInfo
from ..._core.session.tools import is_desktop_session

if TYPE_CHECKING:
    from ..._core.session import Session


@dataclass
class StreamServiceInfo:
    scheme: str
    host: str
    port: int
    path: str
    data_formats: List[str]
    location: str
    transport: str


class StreamCxnConfig(abc.ABC):
    def __init__(
        self,
        infos: Union[List["StreamServiceInfo"], "StreamServiceInfo"],
        protocols: Union[List[str], str],
        transport: str = "websocket",
        api_cfg_key: str = "",
    ):
        if isinstance(infos, list) and len(infos) == 0:
            raise ValueError("infos are empty")

        if not isinstance(infos, list):
            infos = [infos]

        if not isinstance(protocols, list):
            protocols = [protocols]

        self._num_infos = len(infos)
        self._infos = infos
        self._protocols = protocols
        self._transport = transport
        self._available = [True] * len(infos)
        self._index = 0
        self._api_cfg_key = api_cfg_key

    @property
    def transport(self):
        return self._transport

    @property
    def info(self):
        return self._infos[self._index]

    @property
    def num_infos(self) -> int:
        return self._num_infos

    def info_not_available(self) -> None:
        self._available[self._index] = False

    def has_available_info(self) -> bool:
        return any(self._available)

    def next_available_info(self) -> int:
        if not self.has_available_info():
            raise ValueError("No available infos")
        infos = compress(
            self._infos[self._index + 1 :] + self._infos[: self._index],
            self._available[self._index + 1 :] + self._available[: self._index],
        )
        info = next(infos)
        self._index = self._infos.index(info)
        if self._index == 0:
            raise StopIteration()
        return self._index

    @property
    def url(self):
        if hasattr(self.info, "url"):
            url = self.info.url
        else:
            url = self._get_url(self.info)
            self.info.url = url
        return url

    @property
    def url_scheme(self):
        return self.info.scheme

    @property
    def urls(self):
        return [self._get_url(info) for info in self._infos]

    @property
    def headers(self):
        return []

    @property
    def data_formats(self):
        return self.info.data_formats

    @property
    def supported_protocols(self):
        return self._protocols

    def reset_reconnection_config(self):
        self._index = 0

    @property
    def no_proxy(self):
        return ProxyInfo.get_no_proxy()

    @property
    def proxy_config(self):
        proxies_info = ProxyInfo.get_proxies_info()
        if self.url_scheme == "wss":
            # try to get https proxy then http proxy if https not configured
            return proxies_info.get("https", proxies_info.get("http", None))
        else:
            return proxies_info.get("http", None)

    @property
    def data_fmt(self):
        if not self.data_formats:
            return ""
        return self.data_formats[0]

    @abc.abstractmethod
    def _get_url(self, info: "StreamServiceInfo") -> str:
        pass

    def __str__(self) -> str:
        urls = "\n\t\t\t   ".join(self.urls)
        s = (
            f"{self.__class__.__name__}\n"
            f"\t\theaders={self.headers},"
            f"data_formats={self.data_formats},"
            f"supported_protocols={self.supported_protocols},"
            f"no_proxy={self.no_proxy},"
            f"proxy_config={self.proxy_config},"
            f"data_fmt={self.data_fmt}\n"
            f"\t\turls : {urls}"
        )
        return s

    @property
    def api_cfg_key(self):
        return self._api_cfg_key


class DesktopStreamCxnConfig(StreamCxnConfig):
    def __init__(
        self,
        session: "Session",
        infos: Union[List["StreamServiceInfo"], "StreamServiceInfo"],
        protocols: Union[List[str], str],
        api_cfg_key: str = "",
    ):
        super().__init__(infos, protocols, api_cfg_key=api_cfg_key)
        self._session = session

    @property
    def headers(self):
        app_version = os.getenv("DP_PROXY_APP_VERSION")
        headers = [f"x-tr-applicationid: {self._session.app_key}"]

        if self._session._access_token:
            headers.append(f"Authorization: Bearer {self._session._access_token}")

        if app_version and is_desktop_session(self._session):
            headers.append(f"app-version: {app_version}")

        return headers

    def _get_url(self, info: "StreamServiceInfo") -> str:
        return f"{info.scheme}://{info.host}:{info.port}/{info.path}"


class PlatformStreamCxnConfig(StreamCxnConfig):
    def _get_url(self, info: "StreamServiceInfo") -> str:
        if self.transport == "tcp":
            return f"{info.host}:{info.port}"
        else:
            path = info.path or "WebSocket"
            return f"{info.scheme}://{info.host}:{info.port}/{path}"


class NullStreamCxnConfig(StreamCxnConfig):
    def __init__(self):
        StreamCxnConfig.__init__(
            self, [StreamServiceInfo("", "", 0, "", [""], "", "")], []
        )

    def _get_url(self, info):
        return ""



from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions import (
    TurnAdjustment,
)
from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions._enums import (
    StandardTurnPeriod,
)


def test_turn_adjustment():
    # given
    expected_dict = {"standardTurnPeriod": "None"}

    # when
    testing_obj = TurnAdjustment(
        standard_turn_period=StandardTurnPeriod.NONE,
    )

    # then
    assert testing_obj.get_dict() == expected_dict

    assert testing_obj.standard_turn_period == StandardTurnPeriod.NONE



from unittest.mock import patch

import pytest

from refinitiv.data import _configure as configure
from refinitiv.data.delivery._data import _data_provider_factory as factory
from tests.unit.conftest import StubConfigure, StubConfig


def test_get_url_method_return_not_none(content_type):
    # given
    get_url = factory.get_url
    config = configure.get_config()

    # when
    testing_value = get_url(content_type, config)

    # then
    assert testing_value is not None


def test_get_url_method_raise_error_if_invalid_content_type():
    # given
    content_type = None
    get_url = factory.get_url
    config = configure.get_config()

    # then
    with pytest.raises(AttributeError, match="Cannot find api_key"):
        # when
        get_url(content_type, config)


def test_get_url_method_raise_error_if_invalid_config(content_type):
    # given
    get_url = factory.get_url

    api_config_key = factory._get_api_config_key(content_type)
    url_config_key = factory._get_url_config_key(content_type)

    # then
    with pytest.raises(AttributeError, match="Cannot find content_url"):
        # when
        get_url(
            content_type,
            StubConfig({api_config_key: StubConfig({url_config_key: None})}),
        )


def test_make_provider_method_return_not_none(content_type):
    # given
    make_provider = factory.make_provider

    # when
    testing_value = make_provider(content_type)

    # then
    assert testing_value is not None


def test_make_provider_method_raise_error_if_invalid_content_type():
    # given
    content_type = None
    make_provider = factory.make_provider

    # then
    with pytest.raises(
        AttributeError, match="Cannot get data provider by content type"
    ):
        # when
        make_provider(content_type)



from typing import Optional, TYPE_CHECKING

from ...._object_definition import ObjectDefinition
from ._enums import (
    ShiftType,
    ShiftUnit,
)


if TYPE_CHECKING:
    from ......_types import OptStr, OptFloat


class CombinedShift(ObjectDefinition):
    """
    Generates the Cross Currency curves for the definitions provided

    Parameters
    ----------
    amount : float, optional
        Amount of shifting, applied to points depending on shift method selected.<br/>
        can be measured in basis points/percents/future price based points.<br/> also
        can be expressed as multiplier for relative shift type.<br/>
    shift_type : ShiftType, optional
        The type of shifting. the possible values are:   * additive: the amount of
        shifting is added to the corresponding curve point,   * relative: the curve
        point is multiplied by the amount of shifting (e.g., if amount = 1, the curve
        point value will be doubled),   * relativepercent: the curve point is multiplied
        by the amount expressed in percentages (e.g., if amount = 1, the curve point
        value will multiplied by [1+1%]),   * scaled: the curve point is scaled by the
        value of the shifting amount (e.g., if amount = 1.1, the curve point value will
        multiplied by this value).
    shift_unit : ShiftUnit, optional
        The unit that describes the amount of shifting. the possible values are:   * bp:
        the amount of shifting is expressed in basis points,   * percent: the amount of
        shifting is expressed in percentages,   * absolute: the amount of shifting is
        expressed in absolute value.
    end_tenor : str, optional
        The code indicating the end tenor from which the combined shifts scenario is
        applied to curve points. when starttenor equals endtenor:   * shift equals 0 for
        tenor less than starttenor,   * shift equals amount for tenor is equal and
        greater than endtenor. when starttenor less than endtenor:   * shift equals 0
        for tenor which is less or equal starttenor,   * shift rises from 0 to amount in
        period from starttenor to endtenor. shift equals amount for tenor greater than
        endtenor.
    start_tenor : str, optional
        The code indicating the start tenor from which the combined shifts scenario is
        applied to curve points. when starttenor equals endtenor:   * shift equals 0 for
        tenor less than starttenor,   * shift equals amount for tenor is equal and
        greater than endtenor. when starttenor less than endtenor:   * shift equals 0
        for tenor which is less or equal starttenor,   * shift rises from 0 to amount in
        period from starttenor to endtenor. shift equals amount for tenor greater than
        endtenor.
    """

    def __init__(
        self,
        amount: "OptFloat" = None,
        shift_type: Optional[ShiftType] = None,
        shift_unit: Optional[ShiftUnit] = None,
        end_tenor: "OptStr" = None,
        start_tenor: "OptStr" = None,
    ) -> None:
        super().__init__()
        self.amount = amount
        self.shift_type = shift_type
        self.shift_unit = shift_unit
        self.end_tenor = end_tenor
        self.start_tenor = start_tenor

    @property
    def shift_type(self):
        """
        The type of shifting. the possible values are:   * additive: the amount of
        shifting is added to the corresponding curve point,   * relative: the curve
        point is multiplied by the amount of shifting (e.g., if amount = 1, the curve
        point value will be doubled),   * relativepercent: the curve point is multiplied
        by the amount expressed in percentages (e.g., if amount = 1, the curve point
        value will multiplied by [1+1%]),   * scaled: the curve point is scaled by the
        value of the shifting amount (e.g., if amount = 1.1, the curve point value will
        multiplied by this value).
        :return: enum ShiftType
        """
        return self._get_enum_parameter(ShiftType, "shiftType")

    @shift_type.setter
    def shift_type(self, value):
        self._set_enum_parameter(ShiftType, "shiftType", value)

    @property
    def shift_unit(self):
        """
        The unit that describes the amount of shifting. the possible values are:   * bp:
        the amount of shifting is expressed in basis points,   * percent: the amount of
        shifting is expressed in percentages,   * absolute: the amount of shifting is
        expressed in absolute value.
        :return: enum ShiftUnit
        """
        return self._get_enum_parameter(ShiftUnit, "shiftUnit")

    @shift_unit.setter
    def shift_unit(self, value):
        self._set_enum_parameter(ShiftUnit, "shiftUnit", value)

    @property
    def amount(self):
        """
        Amount of shifting, applied to points depending on shift method selected.<br/>
        can be measured in basis points/percents/future price based points.<br/> also
        can be expressed as multiplier for relative shift type.<br/>
        :return: float
        """
        return self._get_parameter("amount")

    @amount.setter
    def amount(self, value):
        self._set_parameter("amount", value)

    @property
    def end_tenor(self):
        """
        The code indicating the end tenor from which the combined shifts scenario is
        applied to curve points. when starttenor equals endtenor:   * shift equals 0 for
        tenor less than starttenor,   * shift equals amount for tenor is equal and
        greater than endtenor. when starttenor less than endtenor:   * shift equals 0
        for tenor which is less or equal starttenor,   * shift rises from 0 to amount in
        period from starttenor to endtenor. shift equals amount for tenor greater than
        endtenor.
        :return: str
        """
        return self._get_parameter("endTenor")

    @end_tenor.setter
    def end_tenor(self, value):
        self._set_parameter("endTenor", value)

    @property
    def start_tenor(self):
        """
        The code indicating the start tenor from which the combined shifts scenario is
        applied to curve points. when starttenor equals endtenor:   * shift equals 0 for
        tenor less than starttenor,   * shift equals amount for tenor is equal and
        greater than endtenor. when starttenor less than endtenor:   * shift equals 0
        for tenor which is less or equal starttenor,   * shift rises from 0 to amount in
        period from starttenor to endtenor. shift equals amount for tenor greater than
        endtenor.
        :return: str
        """
        return self._get_parameter("startTenor")

    @start_tenor.setter
    def start_tenor(self, value):
        self._set_parameter("startTenor", value)



import pytest

from refinitiv.data.content._intervals import (
    Intervals,
    DayIntervalType,
    get_day_interval_type,
)


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        (Intervals.DAILY, DayIntervalType.INTER),
        (
            Intervals.SEVEN_DAYS,
            DayIntervalType.INTER,
        ),
        (Intervals.WEEKLY, DayIntervalType.INTER),
        (Intervals.MONTHLY, DayIntervalType.INTER),
        (
            Intervals.QUARTERLY,
            DayIntervalType.INTER,
        ),
        (
            Intervals.TWELVE_MONTHS,
            DayIntervalType.INTER,
        ),
        (Intervals.YEARLY, DayIntervalType.INTER),
        ("P1Y", DayIntervalType.INTER),
        (
            Intervals.ONE_MINUTE,
            DayIntervalType.INTRA,
        ),
        (
            Intervals.FIVE_MINUTES,
            DayIntervalType.INTRA,
        ),
        (
            Intervals.TEN_MINUTES,
            DayIntervalType.INTRA,
        ),
        (
            Intervals.THIRTY_MINUTES,
            DayIntervalType.INTRA,
        ),
        (
            Intervals.SIXTY_MINUTES,
            DayIntervalType.INTRA,
        ),
        (Intervals.ONE_HOUR, DayIntervalType.INTRA),
        ("PT1H", DayIntervalType.INTRA),
        (
            DayIntervalType.INTRA,
            DayIntervalType.INTRA,
        ),
    ],
)
def test_get_day_interval_type(input_value, expected_value):
    # when
    testing_value = get_day_interval_type(input_value)

    # then
    assert testing_value == expected_value


@pytest.mark.parametrize(
    "input_value",
    [
        "",
        None,
        "None",
    ],
)
def test_get_day_interval_type_raise_error_value(input_value):
    # then
    with pytest.raises(AttributeError, match=f"Value '{input_value}' must be in"):
        # when
        get_day_interval_type(input_value)


@pytest.mark.parametrize("input_value", [[], [[]]])
def test_get_day_interval_type_raise_error_incorrect_day_interval(input_value):
    # then
    with pytest.raises(TypeError, match="Incorrect day interval"):
        # when
        get_day_interval_type(input_value)



import pytest

from refinitiv.data._core.session import (
    DesktopSession,
    PlatformSession,
    GrantPassword,
)


def create_platform_session():
    return PlatformSession(
        app_key="desktop_app_key",
        grant=GrantPassword(username="edp_username", password="edp_password"),
    )


def create_desktop_session():
    return DesktopSession(app_key="desktop_app_key")


@pytest.fixture(
    scope="function", params=[create_platform_session, create_desktop_session]
)
def session(request):
    create_session = request.param
    session = create_session()
    return session



def check_dataframe_contains_instruments(dataframe, expected_instruments):
    retrieved_rows_names = dataframe["Instrument"].tolist()
    for instrument in expected_instruments:
        assert (
            instrument in retrieved_rows_names
        ), f"Expected instrument '{instrument}' is not found in list of rows retrieved from dataframe: {retrieved_rows_names}"


def check_dataframe_contains_fields(dataframe, expected_fields):
    retrieved_column_names = dataframe.columns.array
    retrieved_column_names_lowercase = list(
        map(lambda x: x.lower(), retrieved_column_names)
    )
    for field in expected_fields:
        assert (
            field.lower() in retrieved_column_names_lowercase
        ), f"Expected field '{field}' is not found in list of columns retrieved from dataframe: {retrieved_column_names}"


def check_success_response(df, err):
    assert not df.empty, f"Empty dataframe received"
    assert not err, f"Error received in response: {err}"



import json
from .event import StreamEvent
from .stream_connection import StreamConnection, LOGIN_STREAM_ID
from .stream_cxn_state import StreamCxnState
from ...usage_collection import StreamUsageKey


class OMMStreamConnection(StreamConnection):
    @property
    def subprotocol(self) -> str:
        return "tr_json2"

    def get_login_message(self):
        login_message = {
            "Domain": "Login",
            "ID": LOGIN_STREAM_ID,
            "Key": self._session.get_omm_login_message(),
        }
        return login_message

    def get_close_message(self) -> dict:
        close_message = {
            "Domain": "Login",
            "ID": LOGIN_STREAM_ID,
            "Type": "Close",
        }
        return close_message

    def _handle_login_message(self, message: dict):
        """
        Parameters
        ----------
        message
            {
                'ID': 2,
                'Type': 'Refresh',
                'Domain': 'Login',
                'Key':
                    {
                        'Name': TOKEN_HERE,
                        'Elements': {
                            'AllowSuspectData': 1, 'ApplicationId': '256',
                            'ApplicationName': 'RTO',
                            'AuthenticationErrorCode': 0,
                            'AuthenticationErrorText': {
                                'Type': 'AsciiString', 'Data': None
                            },
                            'AuthenticationTTReissue': 1634562361,
                            'Position': '10.46.188.21/EPUAKYIW3629',
                            'ProvidePermissionExpressions': 1,
                            'ProvidePermissionProfile': 0,
                            'SingleOpen': 1, 'SupportEnhancedSymbolList': 1,
                            'SupportOMMPost': 1,
                            'SupportPauseResume': 0, 'SupportStandby': 0,
                            'SupportBatchRequests': 7,
                            'SupportViewRequests': 1, 'SupportOptimizedPauseResume': 0
                        }
                    },
                'State':
                    {
                        'Stream': 'Open', 'Data': 'Ok',
                        'Text': 'Login accepted by host ads-fanout-lrg-az2-apse1-prd.'
                    }, 'Elements': {'PingTimeout': 30, 'MaxMsgSize': 61426}
            }
        """

        state = message.get("State", {})
        stream_state = state.get("Stream")

        if stream_state == "Open":
            self._state = StreamCxnState.MessageProcessing
            self._connection_result_ready.set()

        elif stream_state == "Closed":
            self.debug(
                f"{self._classname} received a closing message: "
                f"state={self.state}, message={message}"
            )
            self._state = StreamCxnState.Disconnected
            not self.can_reconnect and self._connection_result_ready.set()
            self._config.info_not_available()
            self._listener.close()

        else:
            state_code = state.get("Code", "")
            text = state.get("Text", "")

            if "Login Rejected." in text or state_code == "UserUnknownToPermSys":
                self._config.info_not_available()
                self._listener.close()

                if not self.can_reconnect:
                    self.debug(f"Connection error: {message}")
                    self._state = StreamCxnState.Disconnected
                    self._connection_result_ready.set()

            else:
                raise ValueError(
                    f"{self._classname}._handle_login_message() | "
                    f"Don't know what to do state={self.state}, message={message}"
                )

    def _update_usage_counter(self, stream_id: int, key: dict, message_type: str):
        usage_key = StreamUsageKey(stream_id, key.get("Service"), key.get("Name"))
        self.usage_counter[usage_key][message_type] += 1

    def _process_message(self, message: dict) -> None:
        self.debug(f"{self._classname} process message {message}")
        message_type = message.get("Type")
        stream_id = message.get("ID")
        event = StreamEvent.get(stream_id)

        key = message.get("Key", {})
        self._update_usage_counter(stream_id, key, message_type)

        if message_type == "Refresh":
            self._emitter.emit(event.refresh_by_id, self, message)
            if message.get("Complete", True):
                self._update_usage_counter(stream_id, key, "Complete")
                self._emitter.emit(event.complete_by_id, self, message)

        elif message_type == "Update":
            self._emitter.emit(event.update_by_id, self, message)

        elif message_type == "Status":
            self._emitter.emit(event.status_by_id, self, message)

        elif message_type == "Error":
            # Detect if error is related to Post contrib request,
            # then forward event to post_id listener
            debug_message = message.get("Debug", {}).get("Message")
            if debug_message:
                try:
                    debug_dict = json.loads(debug_message)
                    post_id = debug_dict.get("PostID")
                    if post_id:
                        event = StreamEvent.get(post_id)
                except json.decoder.JSONDecodeError:
                    self.error(f"Cannot decode Debug message as JSON: {debug_message}")
            self._emitter.emit(event.error_by_id, self, message)

        elif message_type == "Ack":
            if stream_id == LOGIN_STREAM_ID:
                self.debug(f"Receive Ack on login ID {stream_id}")
                event = StreamEvent.get(message.get("AckID"))
            else:
                event = StreamEvent.get(stream_id)
            self._emitter.emit(event.ack_by_id, self, message)

        elif message_type == "Ping":
            self.send_message({"Type": "Pong"})

        else:
            raise ValueError(f"Unknown message {message}")



from dataclasses import dataclass
from typing import List, Optional

import pandas as pd


@dataclass
class TopNewsHeadline:
    version_created: str
    headline: str
    story_id: str
    date_line: str
    first_created: str
    snippet: str
    related_headlines: "Optional[List[TopNewsHeadline]]" = None

    @classmethod
    def from_dict(cls, datum: dict) -> "TopNewsHeadline":
        return cls(
            version_created=pd.to_datetime(datum.get("versionCreated")),
            headline=datum.get("text"),
            story_id=datum.get("storyId"),
            date_line=datum.get("dateLine"),
            first_created=pd.to_datetime(datum.get("firstCreated")),
            snippet=datum.get("snippet"),
            related_headlines=[
                cls.from_dict(data) for data in datum.get("relatedHeadlines", [])
            ]
            or None,
        )



import abc
from typing import TYPE_CHECKING, Optional, List, Union, Dict

from pandas import DataFrame

from ...content._df_builder import dfbuilder_rdp, dfbuilder_udf

if TYPE_CHECKING:
    from ...content._df_builder import DFBuilder
    from .._containers import (
        FieldsContainer,
        UniverseContainer,
        ADCDataContainer,
        HPDataContainer,
        CustInstDataContainer,
    )


class Context(abc.ABC):
    def __init__(
        self, universe: "UniverseContainer", fields: "FieldsContainer"
    ) -> None:
        self.fields = fields
        self.universe = universe
        self._data: Optional[list] = []
        self._headers: Optional[List[List[str]]] = None
        self._adc_data: Optional["ADCDataContainer"] = None
        self._hp_data: Optional["HPDataContainer"] = None
        self._cust_inst_data: Optional["CustInstDataContainer"] = None

    def set_data(
        self,
        adc_data: "ADCDataContainer",
        hp_data: "HPDataContainer",
        cust_inst_data: Optional["CustInstDataContainer"] = None,
    ):
        self._adc_data = adc_data
        self._hp_data = hp_data
        self._cust_inst_data = cust_inst_data

    @property
    @abc.abstractmethod
    def dfbuilder(self) -> "DFBuilder":
        # for override
        pass

    @property
    @abc.abstractmethod
    def date_name(self) -> str:
        #  for override
        pass

    @property
    def data(self) -> list:
        return self._data

    @property
    def headers(self) -> List[List[str]]:
        return self._headers

    @property
    @abc.abstractmethod
    def raw(self) -> Optional[Dict]:
        # for override
        pass

    @property
    @abc.abstractmethod
    def df(self) -> Optional[DataFrame]:
        # for override
        pass

    @property
    @abc.abstractmethod
    def can_get_data(self) -> bool:
        # for override
        pass

    @property
    @abc.abstractmethod
    def can_build_df(self) -> bool:
        # for override
        pass

    @abc.abstractmethod
    def build_df(self, use_field_names_in_headers: bool, *args) -> DataFrame:
        # for override
        pass

    @abc.abstractmethod
    def prepare_to_build(
        self, interval, use_field_names_in_headers, df, headers, *args
    ):
        # for override
        pass

    @abc.abstractmethod
    def prepare_data(self, raw, fields, *args) -> list:
        # for override
        pass

    @abc.abstractmethod
    def prepare_headers(self, raw: Union[list, dict], *args) -> List[List[str]]:
        # for override
        pass


class RDPMixin:
    @property
    def dfbuilder(self):
        return dfbuilder_rdp

    @property
    def date_name(self) -> str:
        return "date"


class UDFMixin:
    @property
    def dfbuilder(self):
        return dfbuilder_udf

    @property
    def date_name(self) -> str:
        return "Date"



from typing import Generic, TypeVar, Optional, Callable, TYPE_CHECKING, Any

from ..delivery._data import _data_provider
from ..usage_collection._filter_types import FilterType
from ..usage_collection._logger import get_usage_logger
from ..usage_collection._utils import ModuleName

if TYPE_CHECKING:
    from .._core.session import Session

T = TypeVar("T")


class ContentUsageLoggerMixin(Generic[T]):
    _kwargs: dict
    # Should not change even if class name is changed, should be set by subclasses
    _USAGE_CLS_NAME: str

    def __init__(self, *args, **kwargs):
        get_usage_logger().log_func(
            name=f"{ModuleName.CONTENT}.{self._USAGE_CLS_NAME}.__init__",
            func_path=f"{self.__class__.__module__}."
            f"{self.__class__.__qualname__}."
            f"__init__",
            args=args,
            kwargs=kwargs,
            desc={FilterType.LAYER_CONTENT, FilterType.INIT},
        )
        super().__init__(*args, **kwargs)

    def get_data(
        self,
        session: Optional["Session"] = None,
        on_response: Optional[Callable] = None,
    ) -> T:
        # Library usage logging
        get_usage_logger().log_func(
            name=f"{ModuleName.CONTENT}.{self._USAGE_CLS_NAME}.get_data",
            func_path=f"{self.__class__.__module__}."
            f"{self.__class__.__qualname__}."
            f"get_data",
            kwargs={**self._kwargs},
            desc={FilterType.SYNC, FilterType.LAYER_CONTENT},
        )
        return super().get_data(session, on_response)

    async def get_data_async(
        self,
        session: Optional["Session"] = None,
        on_response: Optional[Callable] = None,
        closure: Optional[Any] = None,
    ) -> T:
        # Library usage logging
        self._kwargs["closure"] = closure
        get_usage_logger().log_func(
            name=f"{ModuleName.CONTENT}.{self._USAGE_CLS_NAME}.get_data_async",
            func_path=f"{self.__class__.__module__}."
            f"{self.__class__.__qualname__}."
            f"get_data_async",
            kwargs={**self._kwargs},
            desc={FilterType.ASYNC, FilterType.LAYER_CONTENT},
        )
        return await super().get_data_async(session, on_response)


class ContentProviderLayer(ContentUsageLoggerMixin, _data_provider.DataProviderLayer):
    def __init__(self, content_type, **kwargs):
        _data_provider.DataProviderLayer.__init__(
            self,
            data_type=content_type,
            **kwargs,
        )



# coding: utf8

from typing import Optional

from .._instrument_definition import ObjectDefinition
from ._enums import (
    BusinessDayConvention,
    DayCountBasis,
    Direction,
    Frequency,
    StubRule,
)


class PremiumLegDefinition(ObjectDefinition):
    """
    API endpoint for Financial Contract analytics,
    that returns calculations relevant to each contract type.

    Parameters
    ----------
    direction : Direction, optional
        The direction of the leg. the possible values are:
        - 'Paid' (the cash flows of the leg are paid to the counterparty),
        - 'Received' (the cash flows of the leg are received from the counterparty).
          Optional for a single leg instrument (like a bond), in that case default value
          is Received. It is mandatory for a multi-instrument leg instrument (like Swap
          or CDS leg).
    notional_ccy : str, optional
        The ISO code of the notional currency. Mandatory if instrument code or
        instrument style has not been defined. In case an instrument code/style has been
        defined, value may comes from the reference data.
    notional_amount : float, optional
        The notional amount of the leg at the period start date. Optional. By default
        1,000,000 is used.
    fixed_rate_percent : float, optional
        The fixed coupon rate in percentage. It is mandatory in case of a single leg
        instrument. Otherwise, in case of multi leg instrument, it can be computed as
        the Par rate.
    interest_payment_frequency : Frequency, optional
        The frequency of the interest payments. Optional if an instrument code/style
        have been defined : in that case, value comes from reference data. Otherwise, it
        is mandatory.
    interest_calculation_method : DayCountBasis, optional
        The Day Count Basis method used to calculate the coupon interest payments.
        Mandatory.
    accrued_calculation_method : DayCountBasis, optional
        The Day Count Basis method used to calculate the accrued interest payments.
        Optional. By default, the same value than interest_calculation_method is used.
    payment_business_day_convention : BusinessDayConvention, optional
        The method to adjust dates to a working day. The possible values are:
        - ModifiedFollowing (adjusts dates according to the Modified Following
          convention - next business day unless is it goes into the next month,
          preceeding is used in that  case),
        - NextBusinessDay (adjusts dates according to the Following convention - Next
          Business Day),
        - PreviousBusinessDay (adjusts dates  according to the Preceeding convention -
          Previous Business Day),
        - NoMoving (does not adjust dates),
        - BbswModifiedFollowing (adjusts dates  according to the BBSW Modified Following
          convention). Optional. In case an instrument code/style has been defined,
          value comes from bond reference data. Otherwise 'ModifiedFollowing' is used.
    first_regular_payment_date : str, optional
        The first regular coupon payment date for leg with an odd first coupon.
        Optional.
    last_regular_payment_date : str, optional
        The last regular coupon payment date for leg with an odd last coupon. Optional.
    payment_business_days : str, optional
        A list of coma-separated calendar codes to adjust dates (e.g. 'EMU' or 'USA').
        Optional. By default the calendar associated to notional_ccy is used.
    stub_rule : StubRule, optional
        The rule that defines whether coupon roll dates are aligned on the  maturity or
        the issue date.  The possible values are:
        - ShortFirstProRata (to create a short period between the start date and the
          first coupon date, and pay a smaller amount of interest for the short
          period.All coupon dates are calculated backward from the maturity date),
        - ShortFirstFull (to create a short period between the start date and the first
          coupon date, and pay a regular coupon on the first coupon date. All coupon
          dates are calculated backward from the maturity date),
        - LongFirstFull (to create a long period between the start date and the second
          coupon date, and pay a regular coupon on the second coupon date. All coupon
          dates are calculated backward from the maturity date),
        - ShortLastProRata (to create a short period between the last payment date and
          maturity, and pay a smaller amount of interest for the short period. All
          coupon dates are calculated forward from the start date). This property may
          also be used in conjunction with first_regular_payment_date and
          last_regular_payment_date; in that case the following values can be defined:
        - Issue (all dates are aligned on the issue date),
        - Maturity (all dates are aligned on the maturity date). Optional. By default
          'Maturity' is used.
    accrued_paid_on_default : bool, optional
        Specifies whether the accrued is paid at the credit event date or not.
        - true :  the accrued is paid at the credit event date
        - false :  the accrued is not paid at the credit event date Optional. Defaults
          to false.
    interest_payment_ccy : str, optional
        The ISO code of the interest payment currency. Mandatory.
    """

    def __init__(
        self,
        *,
        direction: Optional[Direction] = None,
        notional_ccy: Optional[str] = None,
        notional_amount: Optional[float] = None,
        fixed_rate_percent: Optional[float] = None,
        interest_payment_frequency: Optional[Frequency] = None,
        interest_calculation_method: Optional[DayCountBasis] = None,
        accrued_calculation_method: Optional[DayCountBasis] = None,
        payment_business_day_convention: Optional[BusinessDayConvention] = None,
        first_regular_payment_date: Optional[str] = None,
        last_regular_payment_date: Optional[str] = None,
        payment_business_days: Optional[str] = None,
        stub_rule: Optional[StubRule] = None,
        accrued_paid_on_default: Optional[bool] = None,
        interest_payment_ccy: Optional[str] = None,
    ) -> None:
        super().__init__()
        self.direction = direction
        self.notional_ccy = notional_ccy
        self.notional_amount = notional_amount
        self.fixed_rate_percent = fixed_rate_percent
        self.interest_payment_frequency = interest_payment_frequency
        self.interest_calculation_method = interest_calculation_method
        self.accrued_calculation_method = accrued_calculation_method
        self.payment_business_day_convention = payment_business_day_convention
        self.first_regular_payment_date = first_regular_payment_date
        self.last_regular_payment_date = last_regular_payment_date
        self.payment_business_days = payment_business_days
        self.stub_rule = stub_rule
        self.accrued_paid_on_default = accrued_paid_on_default
        self.interest_payment_ccy = interest_payment_ccy

    @property
    def accrued_calculation_method(self):
        """
        The Day Count Basis method used to calculate the accrued interest payments.
        Optional. By default, the same value than interest_calculation_method is used.
        :return: enum DayCountBasis
        """
        return self._get_enum_parameter(DayCountBasis, "accruedCalculationMethod")

    @accrued_calculation_method.setter
    def accrued_calculation_method(self, value):
        self._set_enum_parameter(DayCountBasis, "accruedCalculationMethod", value)

    @property
    def direction(self):
        """
        The direction of the leg. the possible values are:
        - 'Paid' (the cash flows of the leg are paid to the counterparty),
        - 'Received' (the cash flows of the leg are received from the counterparty).
          Optional for a single leg instrument (like a bond), in that case default value
          is Received. It is mandatory for a multi-instrument leg instrument (like Swap
          or CDS leg).
        :return: enum Direction
        """
        return self._get_enum_parameter(Direction, "direction")

    @direction.setter
    def direction(self, value):
        self._set_enum_parameter(Direction, "direction", value)

    @property
    def interest_calculation_method(self):
        """
        The Day Count Basis method used to calculate the coupon interest payments.
        Mandatory.
        :return: enum DayCountBasis
        """
        return self._get_enum_parameter(DayCountBasis, "interestCalculationMethod")

    @interest_calculation_method.setter
    def interest_calculation_method(self, value):
        self._set_enum_parameter(DayCountBasis, "interestCalculationMethod", value)

    @property
    def interest_payment_frequency(self):
        """
        The frequency of the interest payments. Optional if an instrument code/style
        have been defined : in that case, value comes from reference data. Otherwise, it
        is mandatory.
        :return: enum Frequency
        """
        return self._get_enum_parameter(Frequency, "interestPaymentFrequency")

    @interest_payment_frequency.setter
    def interest_payment_frequency(self, value):
        self._set_enum_parameter(Frequency, "interestPaymentFrequency", value)

    @property
    def payment_business_day_convention(self):
        """
        The method to adjust dates to a working day. The possible values are:
        - ModifiedFollowing (adjusts dates according to the Modified Following
          convention - next business day unless is it goes into the next month,
          preceeding is used in that  case),
        - NextBusinessDay (adjusts dates according to the Following convention - Next
          Business Day),
        - PreviousBusinessDay (adjusts dates  according to the Preceeding convention -
          Previous Business Day),
        - NoMoving (does not adjust dates),
        - BbswModifiedFollowing (adjusts dates  according to the BBSW Modified Following
          convention). Optional. In case an instrument code/style has been defined,
          value comes from bond reference data. Otherwise 'ModifiedFollowing' is used.
        :return: enum BusinessDayConvention
        """
        return self._get_enum_parameter(
            BusinessDayConvention, "paymentBusinessDayConvention"
        )

    @payment_business_day_convention.setter
    def payment_business_day_convention(self, value):
        self._set_enum_parameter(
            BusinessDayConvention, "paymentBusinessDayConvention", value
        )

    @property
    def stub_rule(self):
        """
        The rule that defines whether coupon roll dates are aligned on the  maturity or
        the issue date.  The possible values are:
        - ShortFirstProRata (to create a short period between the start date and the
          first coupon date, and pay a smaller amount of interest for the short
          period.All coupon dates are calculated backward from the maturity date),
        - ShortFirstFull (to create a short period between the start date and the first
          coupon date, and pay a regular coupon on the first coupon date. All coupon
          dates are calculated backward from the maturity date),
        - LongFirstFull (to create a long period between the start date and the second
          coupon date, and pay a regular coupon on the second coupon date. All coupon
          dates are calculated backward from the maturity date),
        - ShortLastProRata (to create a short period between the last payment date and
          maturity, and pay a smaller amount of interest for the short period. All
          coupon dates are calculated forward from the start date). This property may
          also be used in conjunction with first_regular_payment_date and
          last_regular_payment_date; in that case the following values can be defined:
        - Issue (all dates are aligned on the issue date),
        - Maturity (all dates are aligned on the maturity date). Optional. By default
          'Maturity' is used.
        :return: enum StubRule
        """
        return self._get_enum_parameter(StubRule, "stubRule")

    @stub_rule.setter
    def stub_rule(self, value):
        self._set_enum_parameter(StubRule, "stubRule", value)

    @property
    def accrued_paid_on_default(self):
        """
        Specifies whether the accrued is paid at the credit event date or not.
        - true :  the accrued is paid at the credit event date
        - false :  the accrued is not paid at the credit event date Optional. Defaults
          to false.
        :return: bool
        """
        return self._get_parameter("accruedPaidOnDefault")

    @accrued_paid_on_default.setter
    def accrued_paid_on_default(self, value):
        self._set_parameter("accruedPaidOnDefault", value)

    @property
    def first_regular_payment_date(self):
        """
        The first regular coupon payment date for leg with an odd first coupon.
        Optional.
        :return: str
        """
        return self._get_parameter("firstRegularPaymentDate")

    @first_regular_payment_date.setter
    def first_regular_payment_date(self, value):
        self._set_parameter("firstRegularPaymentDate", value)

    @property
    def fixed_rate_percent(self):
        """
        The fixed coupon rate in percentage. It is mandatory in case of a single leg
        instrument. Otherwise, in case of multi leg instrument, it can be computed as
        the Par rate.
        :return: float
        """
        return self._get_parameter("fixedRatePercent")

    @fixed_rate_percent.setter
    def fixed_rate_percent(self, value):
        self._set_parameter("fixedRatePercent", value)

    @property
    def interest_payment_ccy(self):
        """
        The ISO code of the interest payment currency. Mandatory.
        :return: str
        """
        return self._get_parameter("interestPaymentCcy")

    @interest_payment_ccy.setter
    def interest_payment_ccy(self, value):
        self._set_parameter("interestPaymentCcy", value)

    @property
    def last_regular_payment_date(self):
        """
        The last regular coupon payment date for leg with an odd last coupon. Optional.
        :return: str
        """
        return self._get_parameter("lastRegularPaymentDate")

    @last_regular_payment_date.setter
    def last_regular_payment_date(self, value):
        self._set_parameter("lastRegularPaymentDate", value)

    @property
    def notional_amount(self):
        """
        The notional amount of the leg at the period start date. Optional. By default
        1,000,000 is used.
        :return: float
        """
        return self._get_parameter("notionalAmount")

    @notional_amount.setter
    def notional_amount(self, value):
        self._set_parameter("notionalAmount", value)

    @property
    def notional_ccy(self):
        """
        The ISO code of the notional currency. Mandatory if instrument code or
        instrument style has not been defined. In case an instrument code/style has been
        defined, value may comes from the reference data.
        :return: str
        """
        return self._get_parameter("notionalCcy")

    @notional_ccy.setter
    def notional_ccy(self, value):
        self._set_parameter("notionalCcy", value)

    @property
    def payment_business_days(self):
        """
        A list of coma-separated calendar codes to adjust dates (e.g. 'EMU' or 'USA').
        Optional. By default the calendar associated to notional_ccy is used.
        :return: str
        """
        return self._get_parameter("paymentBusinessDays")

    @payment_business_days.setter
    def payment_business_days(self, value):
        self._set_parameter("paymentBusinessDays", value)



__all__ = (
    "delete",
    "get",
    "create",
    "create_formula",
    "create_basket",
    "create_udc",
    "Holiday",
    "Basket",
    "Constituent",
    "UDC",
    "VolumeBasedRollover",
    "ManualRollover",
    "DayBasedRollover",
    "SpreadAdjustment",
    "Months",
    "ManualItem",
)
from ._manage import delete, get, create, create_formula, create_basket, create_udc
from ._instrument_prop_classes import (
    Basket,
    Constituent,
    UDC,
    VolumeBasedRollover,
    ManualRollover,
    DayBasedRollover,
    SpreadAdjustment,
    Months,
    ManualItem,
)
from ..ipa.dates_and_calendars.holidays._holidays_data_provider import Holiday



from unittest.mock import patch

import refinitiv.data as rd
from refinitiv.data.delivery._stream.contrib import ContribType
from tests.unit.conftest import StubSession
from tests.unit.delivery.stream.conftest import (
    make_mock_create_offstream_contrib,
)


@patch(
    "refinitiv.data.delivery._stream._stream_factory.create_offstream_contrib",
    new=make_mock_create_offstream_contrib(),
)
def test_omm_stream_contribute_sent():
    session = StubSession(is_open=True)
    rd.session.set_default(session)

    response = rd.delivery.omm_stream.contribute(
        name="TEST",
        fields={"BID": 40.83},
        service="ATS_GLOBAL_1",
        contrib_type=ContribType.UPDATE,
    )

    assert response.is_success is True

    rd.session.set_default(None)


@patch(
    "refinitiv.data.delivery._stream._stream_factory.create_offstream_contrib",
    new=make_mock_create_offstream_contrib(),
)
async def test_omm_stream_contribute_async_sent():
    session = StubSession(is_open=True)
    rd.session.set_default(session)

    response = await rd.delivery.omm_stream.contribute_async(
        name="TEST",
        fields={"BID": 40.83},
        service="ATS_GLOBAL_1",
        contrib_type=ContribType.UPDATE,
    )

    assert response.is_success is True

    rd.session.set_default(None)


@patch(
    "refinitiv.data.delivery._stream._stream_factory.create_offstream_contrib",
    new=make_mock_create_offstream_contrib(was_send=False),
)
def test_omm_stream_contribute_not_sent():
    session = StubSession(is_open=True)
    rd.session.set_default(session)

    def on_ack_callback(ack_msg, stream):
        pass

    def on_error_callback(error_msg, stream):
        pass

    response = rd.delivery.omm_stream.contribute(
        name="TEST",
        fields={"BID": 40.83},
        service="ATS_GLOBAL_1",
        contrib_type=ContribType.UPDATE,
        on_ack=on_ack_callback,
        on_error=on_error_callback,
    )

    assert response.is_success is False

    rd.session.set_default(None)


@patch(
    "refinitiv.data.delivery._stream._stream_factory.create_offstream_contrib",
    new=make_mock_create_offstream_contrib(was_send=False),
)
async def test_omm_stream_contribute_async_not_sent():
    session = StubSession(is_open=True)
    rd.session.set_default(session)

    def on_ack_callback(ack_msg, stream):
        pass

    def on_error_callback(error_msg, stream):
        pass

    response = await rd.delivery.omm_stream.contribute_async(
        name="TEST",
        fields={"BID": 40.83},
        service="ATS_GLOBAL_1",
        contrib_type=ContribType.UPDATE,
        on_ack=on_ack_callback,
        on_error=on_error_callback,
    )

    assert response.is_success is False

    rd.session.set_default(None)


@patch(
    "refinitiv.data.delivery._stream._stream_factory.create_offstream_contrib",
    new=make_mock_create_offstream_contrib(raise_err=True),
)
def test_omm_stream_contribute_raise_error():
    session = StubSession(is_open=True)
    rd.session.set_default(session)

    def on_ack_callback(ack_msg, stream):
        pass

    def on_error_callback(error_msg, stream):
        pass

    response = rd.delivery.omm_stream.contribute(
        name="TEST",
        fields={"BID": 40.83},
        service="ATS_GLOBAL_1",
        contrib_type=ContribType.UPDATE,
        on_ack=on_ack_callback,
        on_error=on_error_callback,
    )

    assert response.is_success is False

    rd.session.set_default(None)


@patch(
    "refinitiv.data.delivery._stream._stream_factory.create_offstream_contrib",
    new=make_mock_create_offstream_contrib(raise_err=True),
)
async def test_omm_stream_contribute_async_raise_error():
    session = StubSession(is_open=True)
    rd.session.set_default(session)

    def on_ack_callback(ack_msg, stream):
        pass

    def on_error_callback(error_msg, stream):
        pass

    response = await rd.delivery.omm_stream.contribute_async(
        name="TEST",
        fields={"BID": 40.83},
        service="ATS_GLOBAL_1",
        contrib_type=ContribType.UPDATE,
        on_ack=on_ack_callback,
        on_error=on_error_callback,
    )

    assert response.is_success is False

    rd.session.set_default(None)



from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions import (
    FxSpotInstrumentDescription,
)
from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions import (
    BidAskFieldsDescription,
    FormulaParameterDescription,
    FxSpotInstrumentDefinition,
)


def test_fx_spot_instrument_description():
    # given
    expected_dict = {
        "fields": {},
        "formula": "string",
        "formulaParameters": [{}],
        "instrumentDefinition": {},
    }

    # when
    testing_obj = FxSpotInstrumentDescription(
        fields=BidAskFieldsDescription(),
        formula_parameters=[FormulaParameterDescription()],
        instrument_definition=FxSpotInstrumentDefinition(),
        formula="string",
    )

    # then
    assert testing_obj.get_dict() == expected_dict

    assert testing_obj.fields == BidAskFieldsDescription()
    assert testing_obj.formula_parameters == [FormulaParameterDescription()]
    assert testing_obj.instrument_definition == FxSpotInstrumentDefinition()
    assert testing_obj.formula == "string"



import re
from typing import TYPE_CHECKING, List, Callable

from .._tools import cached_property
from ..delivery._data._data_provider import ContentValidator

if TYPE_CHECKING:
    from ..delivery._data._data_provider import ParsedData

user_has_no_permissions_expr = re.compile(
    r"TS[A-Z]*\.((Interday)|(Intraday)|(QS))\.UserNotPermission\.[0-9]{5}"
)


class HistoricalContentValidator(ContentValidator):
    @classmethod
    def user_has_permissions(cls, data: "ParsedData") -> bool:
        content_data = data.content_data
        if isinstance(content_data, list) and len(content_data):
            status = content_data[0].get("status", {})
            code = status.get("code", "")

            if status and user_has_no_permissions_expr.match(code):
                data.status["error"] = status
                data.error_codes = code
                data.error_messages = status.get("message")
                return False

        return True

    @classmethod
    def content_data_status_is_not_error(cls, data: "ParsedData") -> bool:
        content_data = data.content_data
        if isinstance(content_data, list) and len(content_data):
            first_content_data = content_data[0]
            status = first_content_data.get("status", {})
            code = status.get("code", "")

            if "Error" in code:
                data.status["error"] = status
                data.error_codes = code
                data.error_messages = status.get("message")

                if not (first_content_data.keys() - {"universe", "status"}):
                    return False

                if "UserRequestError" in code:
                    return True

                return False

        return True

    @cached_property
    def validators(self) -> List[Callable[["ParsedData"], bool]]:
        return [
            self.content_data_is_not_none,
            self.user_has_permissions,
            self.content_data_status_is_not_error,
        ]



from typing import TYPE_CHECKING

from numpy import iterable

from ...._content_provider_layer import ContentProviderLayer
from ....._tools import create_repr, try_copy_to_list
from ..._curves._forward_curve_request_item import ForwardCurveRequestItem
from ....._content_type import ContentType

if TYPE_CHECKING:
    from ..._curves._forward_curve_types import (
        OptStr,
        DefnDefns,
        ExtendedParams,
        CurveDefinition,
        CurveParameters,
        OptForwardCurveDefinitions,
    )


class Definition(ContentProviderLayer):
    """
    Parameters
    ----------
    curve_definition : SwapZcCurveDefinition, optional

    curve_parameters : SwapZcCurveParameters, optional

    forward_curve_definitions : list of ForwardCurveDefinition, optional

    curve_tag : str, optional

    extended_params : dict, optional
        If necessary other parameters.

    Methods
    -------
    get_data(session=session, on_response=on_response)
        Returns a response to the data platform
    get_data_async(session=None, on_response=None, async_mode=None)
        Returns a response asynchronously to the data platform

    Examples
    --------
     >>> import refinitiv.data.content.ipa.curves.forward_curves as forward_curves
     >>> definition = forward_curves.Definition(
     ...     curve_definition=forward_curves.SwapZcCurveDefinition(
     ...         currency="EUR",
     ...         index_name="EURIBOR",
     ...         name="EUR EURIBOR Swap ZC Curve",
     ...         discounting_tenor="OIS",
     ...     ),
     ...     forward_curve_definitions=[
     ...         forward_curves.ForwardCurveDefinition(
     ...             index_tenor="3M",
     ...             forward_curve_tag="ForwardTag",
     ...             forward_start_date="2021-02-01",
     ...             forward_curve_tenors=[
     ...                 "0D",
     ...                 "1D",
     ...                 "2D",
     ...                 "3M",
     ...                 "6M",
     ...                 "9M",
     ...                 "1Y",
     ...                 "2Y",
     ...                 "3Y",
     ...                 "4Y",
     ...                 "5Y",
     ...                 "6Y",
     ...                 "7Y",
     ...                 "8Y",
     ...                 "9Y",
     ...                 "10Y",
     ...                 "15Y",
     ...                 "20Y",
     ...                 "25Y"
     ...             ]
     ...         )
     ...     ]
     ... )
     >>> response = definition.get_data()

     Using get_data_async
     >>> import asyncio
     >>> task = definition.get_data_async()
     >>> response = asyncio.run(task)
    """

    def __init__(
        self,
        curve_definition: "CurveDefinition" = None,
        forward_curve_definitions: "OptForwardCurveDefinitions" = None,
        curve_parameters: "CurveParameters" = None,
        curve_tag: "OptStr" = None,
        extended_params: "ExtendedParams" = None,
    ):
        forward_curve_definitions = try_copy_to_list(forward_curve_definitions)
        request_item = ForwardCurveRequestItem(
            curve_definition=curve_definition,
            forward_curve_definitions=forward_curve_definitions,
            curve_parameters=curve_parameters,
            curve_tag=curve_tag,
        )
        super().__init__(
            content_type=ContentType.FORWARD_CURVE,
            universe=request_item,
            extended_params=extended_params,
        )

    def __repr__(self):
        return create_repr(self)


class Definitions(ContentProviderLayer):
    """
    Parameters
    ----------
    universe : forward_curves.Definition, list of forward_curves.Definition

    Methods
    -------
    get_data(session=session, on_response=on_response)
        Returns a response to the data platform
    get_data_async(session=None, on_response=None, async_mode=None)
        Returns a response asynchronously to the data platform

    Examples
    --------
     >>> import refinitiv.data.content.ipa.curves.forward_curves as forward_curves
     >>>
     >>> forward_curve_definition = forward_curves.Definition(
     ...     curve_definition=forward_curves.SwapZcCurveDefinition(
     ...         currency="EUR",
     ...         index_name="EURIBOR",
     ...         name="EUR EURIBOR Swap ZC Curve",
     ...         discounting_tenor="OIS",
     ...     ),
     ...     forward_curve_definitions=[
     ...         forward_curves.ForwardCurveDefinition(
     ...             index_tenor="3M",
     ...             forward_curve_tag="ForwardTag",
     ...             forward_start_date="2021-02-01",
     ...             forward_curve_tenors=[
     ...                 "0D",
     ...                 "1D",
     ...                 "2D",
     ...                 "3M",
     ...                 "6M",
     ...                 "9M",
     ...                 "1Y",
     ...                 "2Y",
     ...                 "3Y",
     ...                 "4Y",
     ...                 "5Y",
     ...                 "6Y",
     ...                 "7Y",
     ...                 "8Y",
     ...                 "9Y",
     ...                 "10Y",
     ...                 "15Y",
     ...                 "20Y",
     ...                 "25Y"
     ...             ]
     ...         )
     ...     ]
     ... )
     >>> definition = forward_curves.Definitions(
     ...     universe=[forward_curve_definition],
     ... )
     >>> response = definition.get_data()

     Using get_data_async
     >>> import asyncio
     >>> task = definition.get_data_async()
     >>> response = asyncio.run(task)
    """

    def __init__(
        self,
        universe: "DefnDefns",
    ):
        universe = try_copy_to_list(universe)
        if not iterable(universe):
            universe = [universe]

        super().__init__(
            content_type=ContentType.FORWARD_CURVE,
            universe=universe,
            __plural__=True,
        )

    def __repr__(self):
        return create_repr(self, class_name=self.__class__.__name__)



# coding: utf8

from enum import Enum, unique


@unique
class AverageType(Enum):
    ARITHMETIC_RATE = "ArithmeticRate"
    ARITHMETIC_STRIKE = "ArithmeticStrike"
    GEOMETRIC_RATE = "GeometricRate"
    GEOMETRIC_STRIKE = "GeometricStrike"



from enum import Enum, unique


@unique
class ReferenceEntityType(Enum):
    BOND_ISIN = "BondIsin"
    BOND_RIC = "BondRic"
    CHAIN_RIC = "ChainRic"
    ORGANISATION_ID = "OrganisationId"
    TICKER = "Ticker"
